 /*

  This is a part of the LiteStep Shell Source code modified for use with
	PureLS Source code.

  Copyright (C) 1997-98 The LiteStep Development Team

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/


#ifndef  __LSAPI_H
#define __LSAPI_H

/**********************************************************************/

#define LSAPI __declspec( dllexport )

LSAPI INT WINAPI ShellMain( LPCTSTR, HANDLE );

#include "../sdk/include/lsapi.h"
#include "../sdk/include/shelltray.h"

/**********************************************************************/

	#define LSAPI_ABOUTDIALOG  (0xBFFF - 1)  /* MSG    */
	#define   AD_CMD_OPEN     (0)            /* WPARAM */
	#define   AD_CMD_CLOSE    (1)
	#define   AD_TYPE_NORMAL  (0x0000)       /* LPARAM */
	#define   AD_TYPE_DETAIL  (0x0001)
	#define   AD_TYPE_ACTIVE  (0x0100)
	#define   AD_TYPE_TOPMOST (0x0200)

	/* MessageBox MACRO */
	#define ErrorMessageBox(x,y,z) MessageBox(NULL, x, y, z|MB_ICONERROR|MB_TASKMODAL|MB_TOPMOST|MB_SETFOREGROUND)
	#define WarningMessageBox(x, y, z) MessageBox(NULL, x, y, z|MB_ICONWARNING|MB_TASKMODAL|MB_TOPMOST|MB_SETFOREGROUND)

	typedef BOOL (WINAPI *FUNC_BOOL__HWND)(HWND);
	typedef VOID (WINAPI *FUNC_VOID__HWND_BOOL)(HWND, BOOL);
	typedef FARPROC (__stdcall *MS_RunFunc)( HWND, INT, INT, char*, char*, INT );
	typedef FARPROC (__stdcall *MS_WinShutdownFunc)( HWND );

	typedef struct
	{
		HWND* hwnd;
		UINT Message;
		UINT nWndCount;
		//
	} MessageData;

/**********************************************************************/

	/* BANG COMMANDS */

		/* System Commands */
	VOID __cdecl BangLogoff( HWND, LPCTSTR );
	VOID __cdecl BangRun( HWND, LPCTSTR );
	VOID __cdecl BangShutdown( HWND, LPCTSTR );
	VOID __cdecl BangStartup( HWND, LPCTSTR );

		/* Window Managing Commands */
	VOID __cdecl BangCascadeWindows( HWND, LPCTSTR );
	VOID __cdecl BangTileWindowsH( HWND, LPCTSTR );
	VOID __cdecl BangTileWindowsV( HWND, LPCTSTR );

		/* Shell Commands */
	VOID __cdecl BangAbout( HWND, LPCTSTR );
	VOID __cdecl BangExecute( HWND, LPCTSTR );
	VOID __cdecl BangQuit( HWND, LPCTSTR );
	VOID __cdecl BangRecycle( HWND, LPCTSTR );
	VOID __cdecl BangRefresh( HWND, LPCTSTR );
	VOID __cdecl BangLoadModule( HWND, LPCTSTR );
	VOID __cdecl BangUnloadModule( HWND, LPCTSTR );
	VOID __cdecl BangReloadModule( HWND, LPCTSTR );
	VOID __cdecl BangRegisterVar( HWND, LPCTSTR );
	VOID __cdecl BangUnregisterVar( HWND, LPCTSTR );


/**********************************************************************/

	/********************/
	/* Message Handlers */
	/********************/
	LRESULT CALLBACK WndProc( HWND, UINT, WPARAM, LPARAM );
	BOOL PassOnMessage( const UINT, const WPARAM, const LPARAM, const BOOL, LRESULT * const );
	VOID AddWndMessages( const UINT *, const HWND );
	VOID RemoveWndMessages( const UINT *, const HWND );
	VOID PurgeMessages( VOID );
	VOID CleanDeadMessages( VOID );

	/*******************/
	/* Module Handlers */
	/*******************/
	UINT ReloadModule( LPCTSTR const );
	UINT StartModule( LPCTSTR const );
	BOOL AddModules( VOID );
	UINT StopModule( LPCTSTR const );
	VOID RemoveModules( VOID );

	/********************/
	/* Command Handlers */
	/********************/
	VOID PurgeBCs( VOID );

	/******************/
	/* About Handlers */
	/******************/
	VOID LoadAboutDialog( DWORD );
	VOID UnloadAboutDialog( VOID );

	/*******************/
	/* Dialog Handlers */
	/*******************/
	BOOL IsModuleDialogMsg( LPMSG );
	VOID RegisterDialogWnd( HWND );
	VOID UnregisterDialogWnd( HWND );
	VOID RemoveModuleDialogs( VOID );
	VOID CleanDeadDialogs( VOID );

/**********************************************************************/

typedef UINT (WINAPI *SS_Startup)( HANDLE );
typedef BOOL (WINAPI *SS_Enum)(SHTRAYCALLBACKPROC);
typedef BOOL (WINAPI *SS_TrayRegister)(SHTRAYCALLBACKPROC);
typedef BOOL (WINAPI *SS_UnTrayRegister)(SHTRAYCALLBACKPROC);
BOOL CALLBACK TrayCallBack(UINT, WPARAM, LPARAM);



/**********************************************************************/

#endif  /* __LSAPI_H */
