 /*

  This is a part of the LiteStep Shell Source code modified for use with
	PureLS Source code.

  Copyright (C) 1997-98 The LiteStep Development Team

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

	/* Disable "unreferenced formal parameter" compiler warning. */
#ifndef _DEBUG
	#pragma warning ( disable : 4100 )
#endif

	/* Lets just make sure we aren't depending on IE */
#define _WIN32_IE 0x0000

/*** System Includes ***********/
#include <tchar.h>
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <shellapi.h>
/*******************************/

/*** Shell Includes ************/
#include "lsapi.h"
#include "../sdk/include/purels.h"
/*******************************/


VOID VarExpansion(LPTSTR buffer, LPCTSTR value)
{
	_TCHAR tmpBuf[PLS_MAX_SIZE];

	SH_CFG_ExpandBuffer(tmpBuf, value, PLS_MAX_SIZE);

	/* UNSAFE COPY, SHOULD USE SOME EXCEPTION HANDLING */
	_tcscpy(buffer, tmpBuf);
	return;
}

VOID VarExpansionEx(LPTSTR buffer, LPCTSTR value, DWORD dwLength)
{
	SH_CFG_ExpandBuffer(buffer, value, dwLength);
	return;
}

	/* Internal general Tokenize function */
INT Tokenize (LPCTSTR szString, LPTSTR *lpszBuffers, DWORD dwNumBuffers, LPTSTR szExtraParameters, BOOL bToken)
{
	DWORD dwBufferCount = 0;

	if (szString != NULL && *szString != 0)
	{
		_TCHAR buffer[PLS_MAX_SIZE] = {0};
		_TCHAR args[PLS_MAX_SIZE] = {0};

		_tcscpy(args, szString);

		while(
			(dwBufferCount < dwNumBuffers || lpszBuffers == NULL)
			&&
			(bToken ? PL_BreakTokens(buffer, args, args) : PL_SimpleToken(buffer, args, args))
		)
		{
			if (lpszBuffers != NULL)
			{
				_tcscpy(*lpszBuffers++, buffer);
			}
			if (++dwBufferCount < dwNumBuffers && lpszBuffers != NULL)
			{
				*(*lpszBuffers) = 0;
			}
		}

		if (szExtraParameters != NULL)
		{
			_tcscpy(szExtraParameters, args);
		}
	}

	return dwBufferCount;
}

	/*
	This is not identical to the LS version, however the basics are here
	and should be enough for normal use.
	*/
BOOL GetToken( LPCTSTR szString, LPTSTR szToken, LPCTSTR* szNextToken, BOOL useBrackets )
{
	BOOL bResult = FALSE;

	if (szToken != NULL)
	{
		*szToken = 0;
	}

	if (szNextToken != NULL)
	{
		*szNextToken = NULL;
	}

	if (szString != NULL && *szString != 0)
	{
		_TCHAR buffer[PLS_MAX_SIZE] = {0};
		_TCHAR args[PLS_MAX_SIZE] = {0};

		_tcscpy(args, szString);
		PL_StripWS(args);

		if(
			useBrackets != FALSE
			&&
			(
				*args == _T('[') && _tcschr(args, _T(']'))
			) ? PL_BreakTokens(buffer, args, args) : PL_SimpleToken(buffer, args, args)
		)
		{
			bResult = TRUE;

			if (szNextToken != NULL)
			{
				*szNextToken = szString + (_tcslen(szString) - _tcslen(args));

				if (**szNextToken == 0) /* Not really sure if this should happen */
				{
					*szNextToken = NULL;
				}
			}

			if (szToken != NULL)
			{
				_tcscpy(szToken, buffer);
			}
		}
	}

	return bResult;
}

	/*
	New pLS LCTokenize Wrapper

	One difference between new and old LS Tokenizer:
	In the old version, if, and only if, the last
	token ended in a Quotation mark, and dwBufferCount
	is less then dwNumBuffers, then the next buffer
	in *lpszBuffers would be zero terminated with
	zero length.

	In the new version, it is always zero terminated.
	*/
INT LCTokenize (LPCTSTR szString, LPTSTR *lpszBuffers, DWORD dwNumBuffers, LPTSTR szExtraParameters)
{
	return Tokenize(szString, lpszBuffers, dwNumBuffers, szExtraParameters, FALSE);
}

	/*
	New pls CommandTokenize wrapper is completely different... no sloppy error handling.
	It simply returns the contents of each set of [].
	Anything outside of a set of brackets [] is passed over and ignored.
	*/
INT CommandTokenize (LPCTSTR szString, LPTSTR *lpszBuffers, DWORD dwNumBuffers, LPTSTR szExtraParameters)
{
	return Tokenize(szString, lpszBuffers, dwNumBuffers, szExtraParameters, TRUE);
}

VOID CommandParse(LPCTSTR cmd, LPTSTR cmdbuf, LPTSTR argsbuf, DWORD dwCmdBufSize, DWORD dwArgsBufSize)
{
	_TCHAR tmpCmdBuf[PLS_MAX_SIZE] = {0};
	_TCHAR tmpArgsBuf[PLS_MAX_SIZE] = {0};

	if (cmdbuf)
		*cmdbuf = 0;
	if (argsbuf)
		*argsbuf = 0;

	if (PL_SimpleToken( tmpCmdBuf, tmpArgsBuf, cmd )){

		if (cmdbuf && dwCmdBufSize){
			_tcsncpy(cmdbuf, tmpCmdBuf, dwCmdBufSize);
			cmdbuf[dwCmdBufSize-1] = 0;
		}

		if (argsbuf && dwArgsBufSize){
			_tcsncpy(argsbuf, tmpArgsBuf, dwArgsBufSize);
			argsbuf[dwArgsBufSize-1] = 0;
		}
	}
	return;
}
