/*
	This is part of the PureLS Source code, based off of the
	LiteStep Shell Source code, available from www.litestep.net

	Copyright (C) 2000-2003 The PureLS Development Team

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

	/* Disable "unreferenced formal parameter" compiler warning. */
#ifndef _DEBUG
	#pragma warning ( disable : 4100 )
#endif

	/* Lets just make sure we aren't depending on IE */
#define _WIN32_IE 0x0000

/*** System Includes ***********/
#include <tchar.h>
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
/* MinGW doesn't include isspace() and so automaticly */
#include <ctype.h>
/*******************************/

/*** Shell Includes ************/
#include "lsapi.h"
#include "../sdk/include/purels.h"
/*******************************/

typedef struct _LCFILE
{
	UINT nLastLine;
	//
	struct _LCFILE * pNext;
	//
} LCFILE, *PLCFILE;

static PLCFILE s_pLCFiles;

static BOOL _isLCFile( const PLCFILE pFile )
{
	PLCFILE plcf = s_pLCFiles;

	while(NULL != plcf)
	{
		if(pFile == plcf)
		{
			break;
		}

		plcf = plcf->pNext;
	}

	return(NULL != plcf);
}

FILE* LCOpen( LPCTSTR pszPath )
{
	if(NULL != pszPath && 0 != *pszPath)
	{
		return(PL_fOpen(pszPath));
	}
	else
	{
		PLCFILE plcf = (PLCFILE)malloc(sizeof(LCFILE));

		if(NULL != plcf)
		{
			memset(plcf, 0, sizeof(LCFILE));

			plcf->pNext = s_pLCFiles;
			s_pLCFiles = plcf;
		}

		return((FILE*)plcf);
	}
}

BOOL LCClose( FILE * f )
{
	PLCFILE plcf = s_pLCFiles, pprev = NULL;

	while(NULL != plcf)
	{
		if((PLCFILE)f == plcf)
		{
			if(NULL == pprev)
			{
				s_pLCFiles = plcf->pNext;
			}
			else
			{
				pprev->pNext = plcf->pNext;
			}

			free(plcf);

			return(TRUE);
		}

		pprev = plcf;
		plcf = plcf->pNext;
	}

	PL_fClose(f);

	return(TRUE);
}

BOOL LCReadNextCommand( FILE* f, LPTSTR szBuffer, DWORD dwLength )
{
	_TCHAR tmpBuf[PLS_MAX_SIZE];

	if(!_isLCFile((PLCFILE)f))
	{
		while(PL_fGetNextLine(f, 0, tmpBuf, PLS_MAX_SIZE))
		{
			if(!_istpunct(*tmpBuf))
			{
				_tcsncpy(szBuffer, tmpBuf, dwLength);
				szBuffer[dwLength-1] = 0;
				return TRUE;
			}
		}
	}
	else
	{
		PUINT pnLastLine = &((PLCFILE)f)->nLastLine;

		while(SH_CFG_GetLine(pnLastLine, tmpBuf, NULL, PLS_MAX_SIZE, FALSE))
		{
			if(!_istpunct(*tmpBuf))
			{
				_tcsncpy(szBuffer, tmpBuf, dwLength);
				szBuffer[dwLength-1] = 0;
				return TRUE;
			}
		}

		*pnLastLine = 0;
	}

	return FALSE;
}

BOOL LCReadNextConfig( FILE* f, LPCTSTR szPrefix, LPTSTR szBuffer, DWORD dwLength )
{
	_TCHAR szTempPrefix[PLS_MAX_SIZE];

	if(_T('*') != *szPrefix)
	{
		*szTempPrefix = _T('*');
		*(szTempPrefix + 1) = 0;
	}
	else
	{
		*szTempPrefix = 0;
	}

	_tcscat(szTempPrefix, szPrefix);

	if(!_isLCFile((PLCFILE)f))
	{
		return PL_fGetLine(f, 0, szBuffer, szTempPrefix, dwLength);
	}
	else
	{
		PUINT pnLastLine = &((PLCFILE)f)->nLastLine;

		if(SH_CFG_GetLine(pnLastLine, szBuffer, szTempPrefix, dwLength, FALSE))
		{
			return TRUE;
		}

		*pnLastLine = 0;
	}

	return FALSE;
}

BOOL LCReadNextLine( FILE *f, LPTSTR szBuffer, DWORD dwLength )
{
	if(!_isLCFile((PLCFILE)f))
	{
		return PL_fGetNextLine(f, 0, szBuffer, dwLength);
	}
	else
	{
		PUINT pnLastLine = &((PLCFILE)f)->nLastLine;

		if(SH_CFG_GetLine(pnLastLine, szBuffer, NULL, dwLength, FALSE))
		{
			return TRUE;
		}

		*pnLastLine = 0;
	}

	return FALSE;
}
