 /*

  This is a part of the LiteStep Shell Source code modified for use with
	PureLS Source code.

  Copyright (C) 2000 - 2004 The PureLS Development Team
  Copyright (C) 1997-98 The LiteStep Development Team

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

	/* Disable "unreferenced formal parameter" compiler warning. */
#ifndef _DEBUG
	#pragma warning ( disable : 4100 )
#endif

	/* Lets just make sure we aren't depending on IE */
#define _WIN32_IE 0x0000

/*** System Includes ***********/
#include <tchar.h>
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <shellapi.h>
/*******************************/

/*** Shell Includes ************/
#include "lsapi.h"
#include "../sdk/include/purels.h"
/*******************************/

extern HINSTANCE g_hMainInst;


HRGN BitmapToRegion( HBITMAP hBmp, COLORREF clrTranparent, COLORREF clrTolerance, INT nOffsetX, INT nOffsetY )
{
	BITMAP bm32;
	HBITMAP hBmp32 = NULL;
	HRGN hRgn = NULL;

	/*
	 * No reason to do clear bm32, except to get rid of compiler warning about it being
	 * used without being initialized, even though that is not actually impossible.  Dumb compiler. :)
	 * In anycase, I dislike compiler warnings, so I make them go away.
	 */
	memset(&bm32, 0, sizeof(bm32));

	if(NULL != hBmp) /* Are we working for absent-minded module coders? */
	{
		BITMAP bm;
		HDC hMemDC = NULL;

		/* Kind of a silly way to handle an error case, but it works. */
		if(0 != GetObject(hBmp, sizeof(bm), &bm))
		{
			/* Create a memory DC inside which we will scan the bitmap content */
			hMemDC = CreateCompatibleDC(NULL);
		}

		if(NULL != hMemDC)
		{
			/*
			 * NOTE: GetDIBits -could- work in place of the below BitBlt copies, but it gets
			 * really messy later on when trying to read the pixels, because we would have to
			 * translate them into RGB values that we could work with. So just use our BitBlt method.
			 */
			BITMAPINFOHEADER bmpInfoRGB32;
			VOID * pBits32;

			/* Create a 32 bits depth bitmap and select it into the memory DC */
			bmpInfoRGB32.biSize          = sizeof(BITMAPINFOHEADER);
			bmpInfoRGB32.biWidth         = bm.bmWidth;
			bmpInfoRGB32.biHeight        = bm.bmHeight;
			bmpInfoRGB32.biPlanes        = 1;
			bmpInfoRGB32.biBitCount      = 32;
			bmpInfoRGB32.biCompression   = BI_RGB;
			bmpInfoRGB32.biSizeImage     = 0;
			bmpInfoRGB32.biXPelsPerMeter = 0;
			bmpInfoRGB32.biYPelsPerMeter = 0;
			bmpInfoRGB32.biClrUsed       = 0;
			bmpInfoRGB32.biClrImportant  = 0;

			hBmp32 = CreateDIBSection(
				 hMemDC
				,(BITMAPINFO *)&bmpInfoRGB32
				,DIB_RGB_COLORS
				,&pBits32
				,NULL
				,0
			);

			if(NULL != hBmp32)
			{
				HDC hTempDC;

				/* Create a DC just to copy the bitmap into the memory DC */
				hTempDC = CreateCompatibleDC(NULL);

				if(NULL != hTempDC)
				{
					HBITMAP hOldBmp; /* I don't like re-assigning parameter variables, so use a 2nd backup HBITMAP */

					/* Copy the bitmap into the memory DC */
					hBmp32 = (HBITMAP)SelectObject(hMemDC, hBmp32);
					hOldBmp = (HBITMAP)SelectObject(hTempDC, hBmp);

					BitBlt(hMemDC, 0, 0, bm.bmWidth, bm.bmHeight, hTempDC, 0, 0, SRCCOPY);

					/* We are done with the our DCs now */
					SelectObject(hTempDC, hOldBmp);
					hBmp32 = (HBITMAP)SelectObject(hMemDC, hBmp32);

					if(0 != GetObject(hBmp32, sizeof(bm32), &bm32))
					{
						/* If creating our image succeeded, then start our region */
						hRgn = CreateRectRgn(0, 0, 0, 0);
					}

					/* All done with hTempDC, so delete */
					DeleteDC(hTempDC);
				}

				/* We'll delete hBmp32 later */
			}

			/* All done with hMemDC, so delete */
			DeleteDC(hMemDC);
		}

		/* We don't own hBmp, so leave it alone - don't destroy */
	}

	/* If creating our working bitmap image succeeded, then an empty region has been created. */
	if(NULL != hRgn)
	{
		DWORD maxRects;
		HGLOBAL hData;
		RGNDATA *pRgnData;
		HRGN h;
		BYTE *p, *p32;
		RECT *pr;
		INT y, x, x0;

		BYTE clrLR, clrLG, clrLB;
		BYTE clrHR, clrHG, clrHB;
		BYTE tmpR, tmpG, tmpB;

		/*
		 * Get our Transparent pixel ranges.
		 *
		 * Yes we could squish all of this into single lines,
		 * but this, I think, is clearer.
		 */
		clrLR = clrHR = GetRValue(clrTranparent);
		clrLG = clrHG = GetGValue(clrTranparent);
		clrLB = clrHB = GetBValue(clrTranparent);

		tmpR = GetRValue(clrTolerance);
		tmpG = GetGValue(clrTolerance);
		tmpB = GetBValue(clrTolerance);

		clrLR = (BYTE)(clrLR > tmpR ? (clrLR - tmpR) : 0x00);
		clrLG = (BYTE)(clrLG > tmpG ? (clrLG - tmpG) : 0x00);
		clrLB = (BYTE)(clrLB > tmpB ? (clrLB - tmpB) : 0x00);

		clrHR = (BYTE)(tmpR < (0xFF - clrHR) ? (clrHR + tmpR) : 0xFF);
		clrHG = (BYTE)(tmpG < (0xFF - clrHG) ? (clrHG + tmpG) : 0xFF);
		clrHB = (BYTE)(tmpB < (0xFF - clrHB) ? (clrHB + tmpB) : 0xFF);

		/* Get how many bytes per row we have for the bitmap bits (rounded up to 32 bits) */
		while(0 != (bm32.bmWidthBytes % 4))
		{
			bm32.bmWidthBytes++;
		}

		/*
		 * For better performances, we will use the ExtCreateRegion() function to create the
		 * region. This function take a RGNDATA structure on entry. We will add rectangles by
		 * amount of ALLOC_UNIT number in this structure.
		 *
		 * NOTE: This is only true with complex regions.  I'm wondering if the majority of the
		 * regions created are not all that complex.  It seems this approach maintains a small
		 * overhead from the dynamic data allocation until a certain region complexity is
		 * reached.  Only then will the benefit from using this approach be obtained.
		 */
		#define ALLOC_UNIT	300 /* It is about 1,600 bytes allocated per 100 units. */
		maxRects = ALLOC_UNIT;

		hData = GlobalAlloc(GMEM_MOVEABLE, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects));

		pRgnData = (RGNDATA *)GlobalLock(hData);
		pRgnData->rdh.dwSize = sizeof(RGNDATAHEADER);
		pRgnData->rdh.iType = RDH_RECTANGLES;
		pRgnData->rdh.nRgnSize = 0;

		/* Clear our Rectangle Accumulation objects */
		pRgnData->rdh.nCount = 0;
		SetRect(&pRgnData->rdh.rcBound, MAXLONG, MAXLONG, 0, 0);

		/* Scan each bitmap row from bottom to top (the bitmap is inverted vertically) */
		p32 = (BYTE *)bm32.bmBits + (bm32.bmHeight - 1) * bm32.bmWidthBytes;

		for(y = 0; y < bm32.bmHeight; y++)
		{
			/* Scan each bitmap pixel from left to right */
			for(x = 0; x < bm32.bmWidth; x++)
			{
				/* Store current non-transparent position */
				x0 = x;

				/* Search for a continuous range of "non transparent pixels" */
				while(x < bm32.bmWidth)
				{
					/* Temporary pointer for transparent pixel search */
					p = (BYTE*)((ULONG*)p32 + x);

					if(
						*p >= clrLB && *p++ <= clrHB
						&&
						*p >= clrLG && *p++ <= clrHG
						&&
						*p >= clrLR && *p   <= clrHR
					)
					{
						break; /* Pixel is Transparent */
					}

					x++;
				}

				/* If we ran into a transparent pixel, then lets add the previous block of non transparent pixels to our region. */
				if(x != x0)
				{
					/*
					 * NOTE: We should do some fancy check to see what percentage of the image we have scanned,
					 * and what the current size is, to determine how big we should resize it.
					 * Because if we only have a few more pixels to go, no point in allocating a huge block of memory.
					 * Or if we are barely starting the image, and we are already reallocating, then reallocate with
					 * a larger block than normal.  Memmory reallocation is an expensive task.  So minimizing would be good.
					 */
					if(pRgnData->rdh.nCount >= maxRects)
					{
						GlobalUnlock(hData);

						maxRects += ALLOC_UNIT;

						hData = GlobalReAlloc(hData, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects), GMEM_MOVEABLE);
						pRgnData = (RGNDATA *)GlobalLock(hData);

						//DebugPrintf(FALSE, _T("BitmapToRegion"), _T("maxRects = %d - %x"), maxRects, sizeof(RECT)*maxRects);
					}

					/*
					 * Add the pixels (x0, y) to (x, y+1) as a new rectangle in the region.
					 */

					/* Get the address of the RECT located at pRgData->rdh.nCount into the array of RECT */
					pr = &(((RECT *)&pRgnData->Buffer)[pRgnData->rdh.nCount]);
					SetRect(pr, nOffsetX + x0, nOffsetY + y, nOffsetX + x, nOffsetY + y + 1);

					if(pr->left   < pRgnData->rdh.rcBound.left)   pRgnData->rdh.rcBound.left   = pr->left;
					if(pr->top    < pRgnData->rdh.rcBound.top)    pRgnData->rdh.rcBound.top    = pr->top;
					if(pr->right  > pRgnData->rdh.rcBound.right)  pRgnData->rdh.rcBound.right  = pr->right;
					if(pr->bottom > pRgnData->rdh.rcBound.bottom) pRgnData->rdh.rcBound.bottom = pr->bottom;

					pRgnData->rdh.nCount++;

					/*
					 * On Windows98, ExtCreateRegion() may fail if the number of rectangles is too
					 * large (ie: > 4000). Therefore, we have to create the region by multiple steps.
					 *
					 * It'd kinda be sick hitting a region that uses this many RECT.
					 */
					if(2000 == pRgnData->rdh.nCount)
					{
						h = ExtCreateRegion(NULL, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects), pRgnData);
						CombineRgn(hRgn, hRgn, h, RGN_OR);
						DeleteObject(h);

						/* Clear our Rectangle Accumulation objects */
						pRgnData->rdh.nCount = 0;
						SetRect(&pRgnData->rdh.rcBound, MAXLONG, MAXLONG, 0, 0);
					}
				}
			}

			/* Go to next row (remember, the bitmap is inverted vertically) */
			p32 -= bm32.bmWidthBytes;
		}

		/* Extend the region with the remaining rectangles */
		if(0 != pRgnData->rdh.nCount)
		{
			h = ExtCreateRegion(NULL, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects), pRgnData);
			CombineRgn(hRgn, hRgn, h, RGN_OR);
			DeleteObject(h);
		}

		/* Clean up */
		GlobalUnlock(hData);
		GlobalFree(hData);
	}

	/* Ok we are finally done with hBmp32, so delete it */
	if(NULL != hBmp32)
	{
		DeleteObject(hBmp32);
	}

	return(hRgn);
}


// Takes strings of the form:
//   File.bmp
//   .extract
//   .extract=file.exe[,3]
HBITMAP LoadLSImage(LPCTSTR pszImage, LPCTSTR pszFile)
{
	LPTSTR lpPipePos;
	HBITMAP hBitmap = NULL;

	if(NULL == pszImage || 0 == _tcsicmp(pszImage, _T(".none")))
	{
		return NULL;
	}

	// Bitmap merging [originally] by Thedd
	//  Thedd - pic1.bmp|pic2.bmp merges the images. Works recursively,
	//  so pic1.bmp|.extract=whatever.dll,3|pic2.bmp also works etc...
	//  pic1.bmp is put "on top of" pic2.bmp in pic1.bmp|pic2.bmp.
	//  Images should be of the same size, or wierd things will happen.
	lpPipePos = _tcschr(pszImage, _T('|'));

	if(NULL != lpPipePos)
	{
		_TCHAR szBuffer1[MAX_PATH + 15]; // +15 is in case it is in the form of: ".extract=whatever.dll,3"
		_TCHAR szBuffer2[PLS_MAX_SIZE]; // can specify multiple images, so make the buffer large enough
		HBITMAP hTmpBmp1, hTmpBmp2;
		HDC hTmpDC1, hTmpDC2;//, hMemDC;
		INT cx1, cy1, cx2, cy2, cx3, cy3;

		_tcsncpy(szBuffer1, pszImage, lpPipePos - pszImage);
		_tcscpy(szBuffer2, lpPipePos + 1);

		/* Load specified Images */
		hTmpBmp1 = LoadLSImage(szBuffer1, pszFile);
		hTmpBmp2 = LoadLSImage(szBuffer2, pszFile);

		/* If either of them are NULL, then return the other (even though it may be NULL as well) */
		if(NULL == hTmpBmp2)
		{
			return hTmpBmp1;
		}
		if(NULL == hTmpBmp1)
		{
			return hTmpBmp2;
		}

		/* Get some image info */
		GetLSBitmapSize(hTmpBmp1, &cx1, &cy1);
		GetLSBitmapSize(hTmpBmp2, &cx2, &cy2);

		/* Create two compatible DCs to copy images into and work with */
		hTmpDC1 = CreateCompatibleDC(NULL);
		hTmpDC2 = CreateCompatibleDC(NULL);

		/* Select our images into the temp dc */
		hTmpBmp1 = (HBITMAP)SelectObject(hTmpDC1, hTmpBmp1);
		hTmpBmp2 = (HBITMAP)SelectObject(hTmpDC2, hTmpBmp2);

#if 0 /* LS 24.6+ merge style */
		cx3 = max(cx1, cx2);
		cy3 = max(cy1, cy2);

		/* Create a compatible mem dc from the second image */
		hMemDC = CreateCompatibleDC(hTmpDC2);

		/* Create a compatible bitmap from the second image */
		hBitmap = CreateCompatibleBitmap(hTmpDC2, cx3, cy3);

		/* Select our final bitmap object, that we will return, into our mem dc */
		hBitmap = (HBITMAP)SelectObject(hMemDC, hBitmap);

		/*
		 * If the second image isn't the exact size of the final image,
		 * then fill the final image background with the transparent color.
		 */
		if(cx2 != cx3 || cy2 != cy3)
		{
			HBRUSH hbrTrans;
			RECT rClr;

			rClr.left = 0;
			rClr.top = 0;
			rClr.right = cx3;
			rClr.bottom = cy3;

			hbrTrans = CreateSolidBrush(0x00FF00FF);

			FillRect(hMemDC, &rClr, hbrTrans);

			DeleteObject(hbrTrans);
		}

		/* Copy the second image into our mem dc, centered. */
		BitBlt(hMemDC, (cx3 - cx2) / 2, (cy3 - cy2) / 2, cx2, cy2, hTmpDC2, 0, 0, SRCCOPY);

		/* Merge the first bitmap ontop of the second, centered, support transparency */
		TransparentBltLS(hTmpDC2, (cx3 - cx1) / 2, (cy3 - cy1) / 2, cx1, cy1, hTmpDC1, 0, 0, 0x00FF00FF);

		/* Release our image, and return whatever was in there. hBitmap now has our final image in it */
		hBitmap = (HBITMAP)SelectObject(hMemDC, hBitmap);

		/* Get rid of our memory device context */
		DeleteDC(hMemDC);

		/* Release the two working bitmaps and delete them. */
		DeleteObject(SelectObject(hTmpDC1, hTmpBmp1));
		DeleteObject(SelectObject(hTmpDC2, hTmpBmp2));
#else
		cx3 = cx1 >= cx2 ? 0 : ((cx2 - cx1) / 2);
		cy3 = cy1 >= cy2 ? 0 : ((cy2 - cy1) / 2);

		/* Merge the first bitmap ontop of the second, centered, support transparency */
		TransparentBltLS(hTmpDC2, cx3, cy3, min(cx1, cx2), min(cy1, cy2), hTmpDC1, 0, 0, 0x00FF00FF);

		/* Release the two working bitmaps, delete them, and return whatever was in there */
		DeleteObject(SelectObject(hTmpDC1, hTmpBmp1));
		hBitmap = (HBITMAP)SelectObject(hTmpDC2, hTmpBmp2);
#endif

		/* Get rid of our temp device contexts */
		DeleteDC(hTmpDC1);
		DeleteDC(hTmpDC2);
	}
	// End bitmap merging by Thedd
	else if(0 == _tcsnicmp(pszImage, _T(".extract"), _tcslen(_T(".extract"))))
	{
		HICON hIcon;

		hIcon = LoadLSIcon(pszImage, pszFile);

		if(NULL != hIcon)
		{
			hBitmap = BitmapFromIcon(hIcon);
			DestroyIcon(hIcon);
		}
	}
	else	// For now, we only support .BMP files
	{
		_TCHAR szImageFinal[MAX_PATH];
		_TCHAR szFullPath[MAX_PATH];

		SH_CFG_ExpandBuffer(szImageFinal, pszImage, MAX_PATH);

		if(FALSE != LSGetImagePath(szFullPath, MAX_PATH))
		{
			_tcscat(szFullPath, szImageFinal);

			hBitmap = (HBITMAP)LoadImage(
				g_hMainInst,
				szFullPath,
				IMAGE_BITMAP,
				0,
				0,
				LR_DEFAULTCOLOR | LR_LOADFROMFILE
			);
		}

		if(NULL == hBitmap)
		{
			hBitmap = (HBITMAP)LoadImage(
				g_hMainInst,
				szImageFinal,
				IMAGE_BITMAP,
				0,
				0,
				LR_DEFAULTCOLOR | LR_LOADFROMFILE
			);
		}
	}

	return hBitmap;
}


#if 1
/*
 * It is better to create a bitmap using CreateCompatibleBitmap
 * instead of CreateBitmapIndirect for performance reasons.
 */
HBITMAP BitmapFromIcon (HICON hIcon)
{
	ICONINFO info;
	HBITMAP	hBitmap = NULL;

	if(NULL != hIcon && GetIconInfo(hIcon, &info))
	{
		BITMAP bm;

		if(0 != GetObject(info.hbmColor, sizeof (BITMAP), &bm))
		{
			HDC	   hDC, hScreenDC;
			HBRUSH hBrush;

			hScreenDC = GetDC(NULL);

			hDC = CreateCompatibleDC(hScreenDC);
			hBitmap = CreateCompatibleBitmap(hScreenDC, bm.bmWidth, bm.bmHeight);

			ReleaseDC(NULL, hScreenDC);

			if(NULL != hBitmap)
			{
				hBitmap = (HBITMAP)SelectObject(hDC, hBitmap);

				hBrush = CreateSolidBrush(0x00FF00FF);
				DrawIconEx(hDC, 0, 0, hIcon, 0, 0, 0, hBrush, DI_NORMAL);
				DeleteObject(hBrush);

				hBitmap = (HBITMAP)SelectObject(hDC, hBitmap);
			}

			DeleteDC(hDC);
		}

		DeleteObject(info.hbmMask);
		DeleteObject(info.hbmColor);
	}

	return(hBitmap);
}
#else
HBITMAP BitmapFromIcon (HICON hIcon)
{
	ICONINFO info;

	if(NULL != hIcon && GetIconInfo(hIcon, &info))
	{
		HDC		dc;
		HBITMAP	hBitmap, oldBMP;
		HBRUSH	hBrush;
		BITMAP	bitmap;

		dc = CreateCompatibleDC(NULL);

		GetObject(info.hbmColor, sizeof (BITMAP), &bitmap);

		hBitmap = CreateBitmapIndirect (&bitmap);

		oldBMP = (HBITMAP)SelectObject(dc, hBitmap);

		hBrush = CreateSolidBrush(0x00FF00FF);

		DrawIconEx (dc, 0, 0, hIcon, 0, 0, 0, hBrush, DI_NORMAL);

		DeleteObject (hBrush);

		DeleteObject (info.hbmMask);
		DeleteObject (info.hbmColor);

		SelectObject(dc, oldBMP);

		DeleteDC (dc);

		return hBitmap;
	}

	return NULL;
}
#endif

// Takes strings of the form:
//   File.ico
//   libary.icl,3 <- libary extraction in imagesfolder
//   c:\path\     <- icon extraction for path out of desktop.ini
//   .extract
//   .extract=file.exe[,3]  ... and returns an icon
HICON LoadLSIcon (LPCTSTR pszImage, LPCTSTR pszFile)
{
	_TCHAR szIconFile[MAX_PATH];
	_TCHAR szBuffer[PLS_MAX_SIZE];
	HICON hIcon = NULL;
	INT nIcon = 0;

	/* Does our aspiring module author need more coffee? */
	if(NULL == pszImage || 0 == _tcsicmp(pszImage, _T(".none")))
	{
		return NULL;
	}

	/* Special case where we use pszFile instead */
	if(NULL != pszFile && 0 == _tcsicmp(pszImage, _T(".extract")))
	{
		_tcsncpy(szBuffer, pszFile, PLS_MAX_SIZE);
		szBuffer[PLS_MAX_SIZE-1] = 0;
	}
	/* Otherwise, pull out the file and/or path from pszImage */
	else
	{
		LPTSTR pszTemp;

		INT nLen = _tcslen(_T(".extract="));

		if(0 != _tcsnicmp(pszImage, _T(".extract="), nLen))
		{
			nLen = 0;
		}

		_tcsncpy(szBuffer, pszImage + nLen, PLS_MAX_SIZE);
		szBuffer[PLS_MAX_SIZE-1] = 0;

		pszTemp = _tcsrchr(szBuffer, _T(','));

		if(NULL != pszTemp)
		{
			*pszTemp++ = 0;

			if(_istdigit(*pszTemp))
			{
				_ttoi(pszTemp);
			}
		}
	}

	/*
	 * If pszImage specified a folder, then we are to look for "desktop.ini"
	 * in that folder and get the icon file defined in "desktop.ini".
	 */
	if(_T('\\') == *(szBuffer + _tcslen(szBuffer) - 1))
	{
		_tcscat(szBuffer, _T("desktop.ini"));

		GetPrivateProfileString(
			 _T(".ShellClassInfo")
			,_T("IconIndex")
			,_T("0")
			,szIconFile
			,11
			,szBuffer
		);

		szIconFile[10] = 0;

		if(_istdigit(*szIconFile))
		{
			nIcon = _ttoi(szIconFile);
		}

		GetPrivateProfileString(
			 _T(".ShellClassInfo")
			,_T("IconFile")
			,_T("")
			,szIconFile
			,PLS_MAX_SIZE
			,szBuffer
		);
	}
	/* Otherwise, make sure we have the full path to the specified icon file */
	else
	{
		if(NULL == _tcschr(szBuffer, _T('\\')))
		{
			if(FALSE == LSGetImagePath(szIconFile, MAX_PATH))
			{
				*szIconFile = 0;
			}

			_tcscat(szIconFile, szBuffer);
		}
		else
		{
			_tcsncpy(szIconFile, szBuffer, MAX_PATH);
			szIconFile[MAX_PATH-1] = 0;
		}
	}

	/* If this is actually a ".ico" file, then use LoadImage() */
	if(0 == _tcsicmp(szIconFile + _tcslen(szIconFile) - _tcslen(_T(".ico")), _T(".ico")))
	{
		hIcon = (HICON)LoadImage(
			 g_hMainInst
			,szIconFile
			,IMAGE_ICON
			,0
			,0
			,LR_DEFAULTCOLOR | LR_LOADFROMFILE
		);

		nIcon = 0;
	}

	/* If we haven't successfully loaded hIcon yet, try extracting the icon from the given resource */
	if(NULL == hIcon)
	{
		/* NOTE: compare nIcon <= 0 so we don't unnecessarily call ExtractIcon if nIcon is already 0 */
		if(nIcon <= 0 || nIcon >= (INT)ExtractIcon(g_hMainInst, szIconFile, 0xffffffff))
		{
			nIcon = 0;
		}

		hIcon = ExtractIcon(g_hMainInst, szIconFile, nIcon);

		if(1 == (INT)hIcon)
		{
			hIcon = NULL;
		}
	}

	return(hIcon);
}

VOID GetLSBitmapSize( HBITMAP hBitmap, INT *cx, INT *cy )
{
	BITMAP bm;

	if(0 == GetObject(hBitmap, sizeof(bm), &bm))
    {
		*cx = 0;
		*cy = 0;
    }
	else
    {
		*cx = bm.bmWidth;
		*cy = bm.bmHeight;
    }

	return;
}

#if 1
VOID TransparentBltLS(
	 HDC hDestDC
	,INT nDestX, INT nDestY
	,INT nWidth, INT nHeight
	,HDC hSrcDC
	,INT nSrcX, INT nSrcY
	,COLORREF clrTransparent
)
{
	HDC hFinalDC, hMemDC, hMaskDC;
	HBITMAP hFinalBmp, hMemBmp, hMaskBmp;

	// Create copy of the source, which is compatible with the destination
	hMemDC = CreateCompatibleDC(hDestDC);
	hMemBmp = CreateCompatibleBitmap(hDestDC, nWidth, nHeight);
	hMemBmp = (HBITMAP)SelectObject(hMemDC, hMemBmp);

	BitBlt(hMemDC, 0, 0, nWidth, nHeight, hSrcDC, nSrcX, nSrcY, SRCCOPY);

	// Create copy of the destination, so we don't modify the actual
	// destination in steps, which if it is a screen DC would cause flickering.
	hFinalDC = CreateCompatibleDC(hDestDC);
	hFinalBmp = CreateCompatibleBitmap(hDestDC, nWidth, nHeight);
	hFinalBmp = (HBITMAP)SelectObject(hFinalDC, hFinalBmp);

	BitBlt(hFinalDC, 0, 0, nWidth, nHeight, hDestDC, nDestX, nDestY, SRCCOPY);

	// Create monochrome bitmap for the transparency mask
	hMaskDC = CreateCompatibleDC(hDestDC);
	hMaskBmp = CreateBitmap(nWidth, nHeight, 1, 1, NULL);
	hMaskBmp = (HBITMAP)SelectObject(hMaskDC, hMaskBmp);

	// Set the memory DC background color to the transparent color, so
	// when we create our transparency mask from this, the transparent
	// area will be "white", while the rest of the image will be "black".
	SetBkColor(hMemDC, clrTransparent);

	// Create the transparency mask from the memory DC.
	// From what I understand, BitBlt uses the source's background color
	// when doing a SRCCOPY and converting to a monochrome bitmap. Coloring
	// everything that isn't the background color of the source, to black
	// on the destination.  And leaving everything that is the background
	// color of the source, as white on the destination.
	// (See above coment on SetBkColor)
	BitBlt(hMaskDC, 0, 0, nWidth, nHeight, hMemDC, 0, 0, SRCCOPY);

	// Currently the memory DC's BkColor == white, and TextColor == black.
	// If we were to AND the colors right now, it would produce an inverted
	// transparent region.  We want the transparent color to be BLACK,
	// and everything else to be its own color.  Because 'color' AND 'black'
	// will produce black. 'color' AND 'white' will produce 'color'.
	SetBkColor(hMemDC, RGB(0,0,0));
	SetTextColor(hMemDC, RGB(255,255,255));
	// After this BitBlt, hMemDC contains the image minus the transparent
	// areas (The transparent areas are now all black).
	BitBlt(hMemDC, 0, 0, nWidth, nHeight, hMaskDC, 0, 0, SRCAND);

	// Set the foreground to black. See comment above.  This will produce
	// the opposite as above.  With the final image having everything
	// painted black except the transparent areas, leaving those untouched.
	SetBkColor(hFinalDC, RGB(255,255,255));
	SetTextColor(hFinalDC, RGB(0,0,0));
	BitBlt(hFinalDC, 0, 0, nWidth, nHeight, hMaskDC, 0, 0, SRCAND);

	// Combine the foreground with the background.
	//
	// The final image contains only the parts that is not to be painted...
	// aka 'transparent areas'.  Everything else is black.
	//
	// The memory image contains everything that isn't transparent.  Everything
	// that is to be transparent is black.
	//
	// With that as the setup, when these items are OR'd together, it creates
	// one final image from the two parts, since 'color' OR 'black' == 'color'
	BitBlt(hFinalDC, 0, 0, nWidth, nHeight, hMemDC, 0, 0, SRCPAINT);

	// Ok, now that our final image has been created, just copy it over
	// to the destination, then free our resources, and we are done!
	BitBlt(hDestDC, nDestX, nDestY, nWidth, nHeight, hFinalDC, 0, 0, SRCCOPY);

	// Free resources
	DeleteObject(SelectObject(hMaskDC, hMaskBmp));
	DeleteObject(SelectObject(hMemDC, hMemBmp));
	DeleteObject(SelectObject(hFinalDC, hFinalBmp));

	DeleteDC(hMaskDC);
	DeleteDC(hMemDC);
	DeleteDC(hFinalDC);

	return;
}

#else
/*
 * We could double buffer this by drawing to a memory dc, instead of painting
 * right back into the destination DC. That would reduce flickering on modules
 * that don't double buffer their drawing routines.
 *
 * However, it would be extra overhead if the module is already
 * doublebuffering.  So I don't think we should.
 */
VOID TransparentBltLS(
	 HDC hDestDC
	,INT nDestX, INT nDestY
	,INT nWidth, INT nHeight
	,HDC hSrcDC
	,INT nSrcX, INT nSrcY
	,COLORREF clrTransparent
)
{
	HDC locMemDC, maskDC;
	HBITMAP bmpImage, maskBitmap;
	COLORREF clrOldBk, clrOldText;

	// Create copy of the source
	locMemDC = CreateCompatibleDC(hDestDC);
	bmpImage = CreateCompatibleBitmap(hDestDC, nWidth, nHeight);
	bmpImage = (HBITMAP)SelectObject(locMemDC, bmpImage);

	BitBlt(locMemDC, 0,0, nWidth,nHeight, hSrcDC, nSrcX,nSrcY, SRCCOPY);

	SetBkColor(locMemDC, clrTransparent);

	// Create monochrome bitmap for the mask
	maskDC = CreateCompatibleDC(hDestDC);
	maskBitmap = CreateBitmap(nWidth, nHeight, 1, 1, NULL);
	maskBitmap = (HBITMAP)SelectObject(maskDC, maskBitmap);

	// Create the mask from the memory DC
	BitBlt(maskDC, 0,0, nWidth,nHeight, locMemDC, 0,0, SRCCOPY);

	// Set the background in locMemDC to black. Using SRCPAINT with black
	// and any other color results in the other color, thus making
	// black the transparent color
	SetBkColor(locMemDC, RGB(0,0,0));
	SetTextColor(locMemDC, RGB(255,255,255));
	BitBlt(locMemDC, 0,0, nWidth,nHeight, maskDC, 0,0, SRCAND);

	// Set the foreground to black. See comment above.
	clrOldBk = SetBkColor(hDestDC, RGB(255,255,255));
	clrOldText = SetTextColor(hDestDC, RGB(0,0,0));
	BitBlt(hDestDC, nDestX,nDestY, nWidth,nHeight, maskDC, 0,0, SRCAND);

	// Combine the foreground with the background
	BitBlt(hDestDC, nDestX,nDestY, nWidth,nHeight, locMemDC, 0,0, SRCPAINT);

	// Restore the previous colors
	SetBkColor(hDestDC, clrOldBk);
	SetTextColor(hDestDC, clrOldText);

	// Free resources
	DeleteObject(SelectObject(maskDC, maskBitmap));
	DeleteObject(SelectObject(locMemDC, bmpImage));

	DeleteDC(maskDC);
	DeleteDC(locMemDC);

	return;
}
#endif

VOID Frame3D(HDC dc, RECT rect, COLORREF TopColor, COLORREF BottomColor, INT Width)
{
	POINT points[3];
	HPEN OldPen;
	HPEN hPen1 = CreatePen(PS_SOLID, 1, TopColor);
	HPEN hPen2 = CreatePen(PS_SOLID, 1, BottomColor);

	/*
	 * decrement rect size by one.
	 *
	 * we are expecting a rect to be passed in using MS rect sizes
	 * which for example use height 0 to 300, meaning they want 300
	 * pixels in their height. However, actual points are 0 to 299
	 * inclusive.
	 */
	rect.bottom--;
	rect.right--;

	/* save old pen, and load in our left/top color */
	OldPen = (HPEN)SelectObject(dc, hPen1);

	/* continue to draw frames until our width is zero */
	while( Width > 0 )
	{
		/* setup left/top points */
		points[0].x = rect.left;
		points[0].y = rect.bottom;
		points[1].x = rect.left;
		points[1].y = rect.top;
		points[2].x = rect.right;
		points[2].y = rect.top;

		/* paint left/top lines */
		Polyline(dc, points, 3);

		/* load our right/bottom color */
		SelectObject(dc, hPen2);

		/* setup right/bottom points */
		points[0].x = rect.right;
		points[0].y = rect.top;
		points[1].x = rect.right;
		points[1].y = rect.bottom;
		points[2].x = rect.left;
		points[2].y = rect.bottom;

		/* paint right/bottom lines */
		Polyline(dc, points, 3);

		/* reload our left/top color */
		SelectObject(dc, hPen1);

		/* setup values for next iteration */
		Width--;
		InflateRect(&rect, -1, -1);
	}

	/* make sure we clean up */
	SelectObject(dc, OldPen);

	DeleteObject(hPen1);
	DeleteObject(hPen2);

	return;
}
