/*------------------------------------------------------------------------------
	This is part of the PureLS Source code.

	Copyright (C) 2000-2003 The PureLS Development Team

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the Free
    Software Foundation; either version 2 of the License, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details.

    You should have received a copy of the GNU General Public License along with
    this program; if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------
 Lets just make sure we aren't depending on IE
------------------------------------------------------------------------------*/
#define _WIN32_IE 0x0000


/*---System Includes----------------------------------------------------------*/
#include <assert.h>
#include <tchar.h>
#include <windows.h>
#include <process.h>
#include <stdlib.h>
#include <stdio.h>
/*---Shell Includes-----------------------------------------------------------*/
#define NOSTARTUP
#define NOLSAPI
#include "../sdk/include/purels.h"
/*----------------------------------------------------------------------------*/

#ifdef PLS_EXCEPTION_HANDLER

/*---dbghelp.dll----------------------------------------------------------------
 This dll is used to generate much better (correct) stack traces on
 systems that has the dbghelp.dll somewhere where it can be found.

 MS Visual C++
  Including dbghelp.h causes 3 "non-standard warnings".

 MinGW
  Including dbghelp.h doesn't work that great, as it's not part of the
  any of my installed distributions.

 The best solution would probably be to move only the needed bits and
 pieces into our own dbghelp.h replacement header
------------------------------------------------------------------------------*/
#if defined(_DBGHELP_) || defined(_IMAGEHLP64) || defined(IMAGEAPI)
#error Make sure neither dbghelp.h or imagehlp.h is included.
#else
typedef enum {
	AddrMode1616, AddrMode1632, AddrModeReal, AddrModeFlat
} ADDRESS_MODE;

typedef struct _tagADDRESS {
    DWORD			Offset;
    WORD			Segment;
    ADDRESS_MODE	Mode;
} ADDRESS, *PADDRESS;

typedef struct _tagSTACKFRAME {
    ADDRESS			AddrPC;
    ADDRESS			AddrReturn;
    ADDRESS			AddrFrame;
    ADDRESS			AddrStack;
    PVOID			FuncTableEntry;
    DWORD			Params[4];
    BOOL			Far;
    BOOL			Virtual;
    DWORD			Reserved[19];
    ADDRESS			AddrBStore;
} STACKFRAME, *PSTACKFRAME;

typedef BOOL (WINAPI *SYMINITIALIZEPROC)(
	HANDLE		hProcess,
	PSTR		szUserSearchPath,  
	BOOL		fInvadeProcess 
);
typedef BOOL (WINAPI *SYMCLEANUPPROC)(
	HANDLE		hProcess
);
typedef BOOL  (WINAPI *SYMREADPROCESSMEMORYPROC)(
	HANDLE		hProcess,
	DWORD		dwBaseAddress,
	PVOID		pvBuffer,
	DWORD		dwSize,
	PDWORD		pdwNumberOfBytesRead
);
typedef PVOID (WINAPI *SYMFUNCTIONTABLEACCESSPROC)(
	HANDLE	hProcess,
	DWORD	dwBaseAddress
);
typedef DWORD (WINAPI *SYMGETMODULEBASEPROC)(
	HANDLE	hProcess,
	DWORD	dwAddress
);
typedef DWORD (WINAPI *SYMTRANSLATEADDRESSPROC)(
	HANDLE		hProcess,
	HANDLE		hThread,
	PADDRESS	pAddress
);
typedef BOOL (WINAPI *STACKWALKPROC)(
	DWORD						dwMachineType, 
	HANDLE						hProcess, 
	HANDLE						hThread, 
	PSTACKFRAME					pStackFrame, 
	PCONTEXT					pContextRecord, 
	SYMREADPROCESSMEMORYPROC	pReadMemoryRoutine,  
	SYMFUNCTIONTABLEACCESSPROC	pFunctionTableAccessRoutine,
	SYMGETMODULEBASEPROC		pGetModuleBaseRoutine, 
	SYMTRANSLATEADDRESSPROC		pTranslateAddress 
);

typedef struct _tagIMAGEHLP_SYMBOL {
	DWORD	dwSizeOfStruct;
	DWORD	dwAddress;
	DWORD	dwSize;
	DWORD	dwFlags;
	DWORD	dwMaxNameLength;
	CHAR	szName[1]; /* variable length */
} IMAGEHLP_SYMBOL, *PIMAGEHLP_SYMBOL;

typedef BOOL (WINAPI *SYMGETSYMFROMADDRPROC)(
	HANDLE				hProcess,
	DWORD				dwAddress,
	PDWORD				pdwDisplacement,
	PIMAGEHLP_SYMBOL	pSymbol
);
#endif /* _DBGHELP_ */





/*---Local Defines------------------------------------------------------------*/

DWORD TLSID_ExceptionHandlers = 0;

typedef struct _PLS_EXCEPTIONCHAIN
{
	struct _PLS_EXCEPTIONCHAIN*	pNext;
	PLS_ExceptionHandler		Handler;
	ULONG						StackPointer;
} PLS_EXCEPTIONCHAIN, *PPLS_EXCEPTIONCHAIN;

#define _QUOTE(x) #x
#define QUOTE(x) _QUOTE(x)
#define __FILE__LINE__ __FILE__ "(" QUOTE(__LINE__) ")"

#ifdef _DEBUG
#define DebugMessage(x) \
	OutputDebugString(_T("[" __FILE__LINE__ "] " x "\r\n"))
#else
#define DebugMessage(x)
#endif

/*----------------------------------------------------------------------------*/



/*--------PLS_ExceptionFilterProc-----------------------------------------------
 Purpose:
 - This should be called whenever an exception happens anywhere in the PureLS
   application domain. This function will forward any processing to delegates
   for those cases where it might do any good.
 - It also provides a mechanism for better exception handling. For every thread
   in the application it holds a list of PLS_ExceptionHandlers that could
   process the error in a better way.
------------------------------------------------------------------------------*/
LONG CALLBACK PLS_ExceptionFilterProc(
	EXCEPTION_POINTERS *pExceptionData
);

/*--------LoadExceptionHandler--------------------------------------------------
 Purpose:
 - Set up the exception handler to process exceptions inside PureLs
   application domain.
------------------------------------------------------------------------------*/
BOOL PLS_LoadExceptionHandler(
	VOID
);

/*--------UnloadExceptionHandler------------------------------------------------
 Purpose:
 - To revert all changes made by LoadExceptionHandler.
------------------------------------------------------------------------------*/
VOID PLS_UnloadExceptionHandler(
	VOID
);

/*--------PLS_StackTraceLookup--------------------------------------------------
 Purpose:
 - This method allocates a linked list which forms the stack trace from
   the specified position down until it reaches the Windows core.
------------------------------------------------------------------------------*/
PPLS_STACKTRACE PLS_StackTraceLookup(
	PCONTEXT pContext /* context specifying where to begin tracing. */
);

/*--------PLS_StackTraceFree----------------------------------------------------
 Purpose:
 - This method frees the linked list allocated with PLS_StackTraceDump.
------------------------------------------------------------------------------*/
VOID PLS_StackTraceFree(
	PPLS_STACKTRACE pFree
);

/*--------PLS_ExceptionHandlerPush----------------------------------------------
 Purpose:
 - This method is used to push another PureLs exception handler, so that
   it may be used to stop exceptions.
------------------------------------------------------------------------------*/
VOID PLS_ExceptionHandlerPush(
	PLS_ExceptionHandler Handler
);

/*--------PLS_ExceptionHandlerPop-----------------------------------------------
 Purpose:
 - This method is used to remove the last exception handler pushed.
------------------------------------------------------------------------------*/
VOID PLS_ExceptionHandlerPop(
	VOID
);



/*----------------------------------------------------------------------------*/


/*--------PLS_StackTraceI386----------------------------------------------------
 Purpose:
 - Poor mans version of stack trace mechanism... This doesn't produce 100%
   correct values all the time, but should be close enough for most operations.
------------------------------------------------------------------------------*/
PPLS_STACKTRACE PLS_StackTraceI386(
	PCONTEXT pContext           /* context specifying where to begin tracing. */
)
{
	PLS_STACKFRAME frame;                /* convinient local storage variable */

	PPLS_STACKTRACE pTop = 0,                        /* top of the stacktrace */
	                pCurrent = 0,          /* current entry in the stacktrace */
	                pPrevious = 0;            /* previous entry in stacktrace */

	frame.ulReturnAddress = 0;
	frame.ulFramePointer = pContext->Ebp;
	frame.ulStackPointer = pContext->Esp;
	frame.ulFunctionAddress = pContext->Eip;
	frame.szFunctionName = 0;
	frame.hModule = 0;

	/*--------------------------------------------------------------------------
	 Go through the stack trace as long as trace.FramePointer is both DWORD
	 aligned and is valid pointer for reading.
	--------------------------------------------------------------------------*/
	while(
		!(
			(frame.ulFramePointer & 0x03)
			||
			IsBadReadPtr((LPCVOID)frame.ulFramePointer, 8)
		 )
	)
	{
		ULONG* pCallOperand;
		MEMORY_BASIC_INFORMATION mbi;

		frame.ulReturnAddress = *((ULONG*)frame.ulFramePointer + 1);

		/*----------------------------------------------------------------------
		 The allocation base for the callers address is always the same as the
		 hModule/hInstance for the module it's contained in.
		----------------------------------------------------------------------*/
		VirtualQuery((LPCVOID)frame.ulFunctionAddress, &mbi, sizeof(mbi));
		frame.hModule = (HMODULE)mbi.AllocationBase;

		/*----------------------------------------------------------------------
		 Save the stack trace in a linked list.
		----------------------------------------------------------------------*/
		pCurrent = (PPLS_STACKTRACE)malloc(sizeof(PLS_STACKTRACE));
		if(!pCurrent)
		{
			DebugMessage("No more memory for stacktrace");
			break;
		}

		if(pPrevious != 0)
		{
			pPrevious->pOuterLevel = pCurrent;
		}
		else
		{
			pTop = pCurrent;
		}

		memcpy(&pCurrent->StackFrame, &frame, sizeof(frame));

		pPrevious = pCurrent;

		/*----------------------------------------------------------------------
		 Deep inside Windows the hModule will be zero, and we're not
		 interested in debugging the Windows core, are we?
		----------------------------------------------------------------------*/
		if(frame.hModule == 0)
		{
			break;
		}

		/* Get frame pointer for next entry in stack trace */
		frame.ulFramePointer = *((ULONG*)frame.ulFramePointer);

		/*----------------------------------------------------------------------
		 When calling a function, the return value will be pushed onto the
		 stack. And then the first thing inside the routine is pushing the base
		 pointer aswell onto the stack. We want the stackpointer as it was
		 before the call, and therefore we add 2*4 to the FramePointer from
		 inside the function. This will not work for pseudo-frames and other
		 "nice" features...

		 TODO: Make it work, always...
		----------------------------------------------------------------------*/
		frame.ulStackPointer = frame.ulFramePointer + 8;

		/*----------------------------------------------------------------------
		 I don't know if that will always be the case, but from what I've seen a
		 call to MessageLoop becomes the equivalent of:
		     __asm call MessageLoop
		 which in turn goes into the form:
		     0xE8 0xAA 0xBB 0xCC 0xDD
		 where, 0xDDCCBBAA is the byte offset to MessageLoop

		 REMARK: If this is indeed true, jumping to CallerAddress-1
		         would actually result in being in the process of
				 calling that function again.

		 TODO: FIXME: Check for the caller address "the right way", whatever
		              that might be... probable "right way", is looking in the
		              section header for function symbols with addresses close
					  to the return address of the previous call, for now, that
		              is to much work though...
		----------------------------------------------------------------------*/
		pCallOperand = ((ULONG*)frame.ulReturnAddress-1);

		if(!IsBadReadPtr(pCallOperand, 4))
		{
			frame.ulFunctionAddress = frame.ulReturnAddress + *pCallOperand;

			if(pPrevious != 0)
			{
				/*--------------------------------------------------------------
				Check whether it's actually one of the cases described in
				the comment above. If so, then the new function should not
				be very far from the return address of the previous
				stack trace entry...
				--------------------------------------------------------------*/
				if(
					8192 <= (
						pPrevious->StackFrame.ulReturnAddress
						- 
						frame.ulFunctionAddress
					)
				)
				{
					frame.ulFunctionAddress = \
						pPrevious->StackFrame.ulReturnAddress;
				}
			}
		}
	}

	if(pCurrent)
	{
		pCurrent->pOuterLevel = 0;
	}

	return(pTop);
}


/*--------PLS_StackTraceDebugHelp-----------------------------------------------
 Purpose:
 - Rich mans stack tracing method
------------------------------------------------------------------------------*/
PPLS_STACKTRACE PLS_StackTraceDebugHelp(
	PCONTEXT pContext           /* context specifying where to begin tracing. */
)
{
	HANDLE hProcess;
	STACKFRAME sf;                       /* used by dbghelp.dll stackwalking  */

	/* routines to be loaded from dbghelp.dll */
	SYMINITIALIZEPROC			_SymInitialize;
	SYMCLEANUPPROC				_SymCleanup;
	STACKWALKPROC				_StackWalk;
	SYMFUNCTIONTABLEACCESSPROC	_SymFunctionTableAccess;
	SYMGETMODULEBASEPROC		_SymGetModuleBase;
	SYMGETSYMFROMADDRPROC		_SymGetSymFromAddr;

	PPLS_STACKTRACE pTop = 0,                        /* top of the stacktrace */
	                pCurrent = 0,               /* current entry in the trace */
	                pPrevious = 0;            /* previous entry in stacktrace */

	/*--------------------------------------------------------------------------
	 First we try to setup all functions and whatnot necessary.
	 NOTE: dbghelp.dll loaded in PLS_StackTraceLookup 
	--------------------------------------------------------------------------*/
	HMODULE hDebugHelp = GetModuleHandle(_T("dbghelp.dll"));

	_SymInitialize = (SYMINITIALIZEPROC)GetProcAddress(
		 hDebugHelp
		,"SymInitialize"
	);

	_SymCleanup = (SYMCLEANUPPROC)GetProcAddress(hDebugHelp, "SymCleanup");
	_StackWalk = (STACKWALKPROC)GetProcAddress(hDebugHelp, "StackWalk");

	_SymFunctionTableAccess = (SYMFUNCTIONTABLEACCESSPROC)GetProcAddress(
		 hDebugHelp
		,"SymFunctionTableAccess"
	);

	_SymGetModuleBase = (SYMGETMODULEBASEPROC)GetProcAddress(
		 hDebugHelp
		,"SymGetModuleBase"
	);

	_SymGetSymFromAddr = (SYMGETSYMFROMADDRPROC)GetProcAddress(
		 hDebugHelp
		,"SymGetSymFromAddr"
	);


	if(
		NULL == _SymInitialize
		||
		NULL == _SymCleanup
		||
		NULL == _StackWalk
		||
		NULL == _SymFunctionTableAccess
		||
		NULL == _SymGetModuleBase
		||
		NULL == _SymGetSymFromAddr
	)
	{
		return(0);
	}

	hProcess = GetCurrentProcess();

	if(!_SymInitialize(hProcess, 0, TRUE))
	{
		/*----------------------------------------------------------------------
		 In win 9X/ME it should be the process identifier, so not knowing
		 whether we're in win9X or not, try that
		----------------------------------------------------------------------*/
		hProcess = (HANDLE)GetCurrentProcessId();

		if(!_SymInitialize(hProcess, 0, TRUE))
		{
			return(0);
		}
	}

	/* Set initial values (much like in the I386 version) */
	memset(&sf, 0, sizeof(sf));
	sf.AddrPC.Offset    = pContext->Eip;
	sf.AddrPC.Mode      = AddrModeFlat;
	sf.AddrStack.Offset = pContext->Esp;
	sf.AddrStack.Mode   = AddrModeFlat;
	sf.AddrFrame.Offset = pContext->Ebp;
	sf.AddrFrame.Mode   = AddrModeFlat;


	/*--------------------------------------------------------------------------
	 We got past initialization, phew...
	--------------------------------------------------------------------------*/
	while(
		_StackWalk(
			 IMAGE_FILE_MACHINE_I386
			,hProcess
			,GetCurrentThread()
			,&sf
			,pContext
			,0
			,_SymFunctionTableAccess
			,_SymGetModuleBase
			,0
		)
		&&
		0 != sf.AddrFrame.Offset
	)
	{
		BYTE symbolBuffer[sizeof(IMAGEHLP_SYMBOL)+sizeof(CHAR)*512];
		DWORD dwDisplacement;
		ULONG* pCallOperand = ((ULONG*)sf.AddrReturn.Offset - 1);
		PIMAGEHLP_SYMBOL pSymbol = (PIMAGEHLP_SYMBOL)symbolBuffer;

		/*----------------------------------------------------------------------
		 Save the stack trace in a linked list.
		----------------------------------------------------------------------*/
		pCurrent = (PPLS_STACKTRACE)malloc(sizeof(PLS_STACKTRACE));

		if(NULL == pCurrent)
		{
			DebugMessage("No more memory for stacktrace");
			break;
		}

		if(NULL != pPrevious)
		{
			pPrevious->pOuterLevel = pCurrent;
		}
		else
		{
			pTop = pCurrent;
		}

		pCurrent->StackFrame.hModule = (HMODULE)_SymGetModuleBase(
			 hProcess
			,sf.AddrPC.Offset
		);
		pCurrent->StackFrame.ulFramePointer = sf.AddrFrame.Offset;
		pCurrent->StackFrame.ulReturnAddress = sf.AddrReturn.Offset;
		pCurrent->StackFrame.ulStackPointer = sf.AddrStack.Offset;

		if(_SymGetSymFromAddr(
			hProcess
			,sf.AddrPC.Offset
			,&dwDisplacement
			,pSymbol
		))
		{
			pCurrent->StackFrame.ulFunctionAddress = 
				sf.AddrPC.Offset + dwDisplacement;

			pCurrent->StackFrame.szFunctionName = (LPCTSTR)malloc(
				sizeof(CHAR) * (strlen(pSymbol->szName) * 1)
			);

			if(NULL != pCurrent->StackFrame.szFunctionName)
			{
#ifdef _UNICODE
				MultiByteToWideChar(
					 CP_ACP
					,0
					,pCurrent->StackFrame.szFunctionName
					,-1
					,(LPCSTR)pSymbol->szName
					,64
				);
#else
				strcpy(
					 (LPSTR)pCurrent->StackFrame.szFunctionName
					,(LPCSTR)pSymbol->szName
				);
#endif
			}
		}
		else
		{
			/*------------------------------------------------------------------
			FIXME: Check I386 variant to know where my hat is...
			------------------------------------------------------------------*/
			pCurrent->StackFrame.ulFunctionAddress = (
				IsBadReadPtr(pCallOperand, 4)
				?
				sf.AddrPC.Offset
				: 
				*pCallOperand + pCurrent->StackFrame.ulReturnAddress
			);

			pCurrent->StackFrame.szFunctionName = NULL;
		}

		pPrevious = pCurrent;

		/*----------------------------------------------------------------------
		 Deep inside Windows the hModule will be zero, and we're not interested
		 in debugging the Windows core, are we?
		----------------------------------------------------------------------*/
		if(0 == pCurrent->StackFrame.hModule)
		{
			break;
		}
	}

	if(NULL != pCurrent)
	{
		pCurrent->pOuterLevel = 0;
	}

	return(pTop);
}

/*--------PLS_StackTraceLookup--------------------------------------------------
 Purpose:
 - This method allocates a linked list which forms the stack trace from the
   specified position down until it reaches the Windows core.
------------------------------------------------------------------------------*/
PPLS_STACKTRACE PLS_StackTraceLookup(
	PCONTEXT pContext           /* context specifying where to begin tracing. */
)
{
	PPLS_STACKTRACE pTrace = 0;
	HMODULE hDebugHelp = LoadLibrary(_T("dbghelp.dll"));

	if(NULL != hDebugHelp)
	{
		pTrace = PLS_StackTraceDebugHelp(pContext);
		FreeLibrary(hDebugHelp);
	}
	
	/* always fall back on I386 solution when no result from dbghelp */
	if(NULL == pTrace)
	{
		pTrace = PLS_StackTraceI386(pContext);
	}

	return(pTrace);
}


/*--------PLS_StackTraceFree----------------------------------------------------
 Purpose:
 - This method frees the linked list allocated with PLS_StackTraceDump.
------------------------------------------------------------------------------*/
VOID PLS_StackTraceFree(
	PPLS_STACKTRACE pFree
)
{
	while(pFree)
	{
		PPLS_STACKTRACE pTemp = pFree;
		pFree = pFree->pOuterLevel;

		if(0 != pTemp->StackFrame.szFunctionName)
		{
			free((PVOID)pTemp->StackFrame.szFunctionName);
			pTemp->StackFrame.szFunctionName = 0;
		}

		free(pTemp);
		pTemp = 0;
	}
}

/*--------PLS_ExceptionHandlerPush----------------------------------------------
 Purpose:
 - This method is used to push another PureLs exception handler, so that it may
   be used to stop exceptions.
------------------------------------------------------------------------------*/
VOID PLS_ExceptionHandlerPush(
	PLS_ExceptionHandler Handler
)
{
	ULONG ulStackPointer = 0;

	/* Setup new top-of-the-chain exception handler */
	PPLS_EXCEPTIONCHAIN pChain = (PPLS_EXCEPTIONCHAIN)malloc(
		sizeof(PLS_EXCEPTIONCHAIN)
	);

	if(NULL == pChain)
	{
		return;
	}

	pChain->pNext = (PPLS_EXCEPTIONCHAIN)TlsGetValue(TLSID_ExceptionHandlers);
	pChain->Handler = Handler;

	/*--------------------------------------------------------------------------
	 Get current stack pointer, so that when an exception happens, we try not to
	 go to into unwanted handlers, and make sure that when an exception actually
	 happens.  When this is executed, Handler, Current Function and the previous
	 Frame Pointer have been pushed to the stack, therefore increment
	 3 * sizeof(ULONG), so that stack points to where it was when the method
	 PLS_ExceptionHandlerPush() was called.

	 TODO: Really allocated the pChain object on the stack, so we don't have to
	       free it, and this pointer wouldn't be necessary.
	--------------------------------------------------------------------------*/
#if _MSC_VER > 0

	__asm {
		mov eax, esp
		add eax, 12 /* (3 * sizeof(ULONG)) */
		mov ulStackPointer, eax
	}

#elif defined(__GNUC__)

	__asm__ (
		"mov %%esp, %0\n"
		"add $12, %0\n"
		: "=r" (ulStackPointer)
		:
	);

#else

	#error Unsupported compiler

#endif

	/* if high enough on the stack, we might wrap */
	if(ulStackPointer > (2 * 3 * sizeof(ULONG)))
	{
		pChain->StackPointer = ulStackPointer;
	}
	else
	{
		pChain->StackPointer = 0xFFFFFFFF;
	}

	/* Insert it as the value stored in the thread */
	TlsSetValue(TLSID_ExceptionHandlers, (PVOID)pChain);

	return;
}

/*--------PLS_ExceptionHandlerPop-----------------------------------------------
 Purpose:
 - This method is used to remove the last exception handler pushed.
------------------------------------------------------------------------------*/
VOID PLS_ExceptionHandlerPop(
	VOID
)
{
	/* Get the top of the stack of exception handlers for thread. */
	PPLS_EXCEPTIONCHAIN pChain = (PPLS_EXCEPTIONCHAIN)TlsGetValue(
		TLSID_ExceptionHandlers
	);

	if(pChain)
	{
		/* Insert it as the value stored in the thread */
		TlsSetValue(TLSID_ExceptionHandlers, (PVOID)pChain->pNext);

		free(pChain);
	}

	return;
}


typedef enum _QIT_INFOCLASS
{
	ThreadEntryProc = 9
} QIT_INFOCLASS;

typedef LONG (WINAPI *QueryInformationThread)(
    HANDLE hThread, QIT_INFOCLASS nInfoClass,
	PVOID pResult, ULONG uInfoSize, PULONG puResultSize
);

/*--------PLS_ExceptionDefaultHandler-------------------------------------------
 Purpose:
 - This method gets called whenever an exception happens that is considered
   possible to fix, but is not fixed by any of the handlers registered from
   outside the main exe.
 - This will process the exception as if it's a "thread-killer", and it should
   ask the user whether it would be a good idea to either end the thread in
   question, or to try to restart it, or just not process the exception and let
   windows take care of it.
------------------------------------------------------------------------------*/
BOOL PLS_ExceptionDefaultHandler(
	 EXCEPTION_POINTERS*	pExceptionData
	,PPLS_STACKTRACE pTrace
)
{
	LONG lSuccess;
	ULONG ulThreadEntryProc;
	QueryInformationThread NtQueryInformationThread;
	int nUserSelection;

	DebugMessage("Try to fix the exception with default handler");

	/*--------------------------------------------------------------------------
	 The following should be done:
	 * Notify the user the current thread has gone bad, let user decide what to
	   do about the exception. (give nice info to do so)
	   a) Try to mend the problem, by restarting the faulty thread.
	   b) Try to mend the problem, just quit the thread and continue.
	   c) Don't try to mend the problem. (Windows will quit the process)
	--------------------------------------------------------------------------*/
	nUserSelection = MessageBox(
		 0
		,_T("An exception has been encountered in PureLS that could\r\n"
		    "NOT be fixed by our internal exception handlers.\r\n"
		    "\r\n"
		    "This is a very serious error.\r\n"
		    "\r\n"
		    "However we would like to give you as a powerful PureLS\r\n"
		    "user one last resort which might fix the problem. The\r\n"
		    "following choices may allow you to recover long enough\r\n"
		    "to save your data and restart your system.\r\n"
		    "\r\n"
		    "Would you like PureLS to try to mend the exception?\r\n"
		    "\r\n"
		    "- Selecting \"Yes\" will make PureLS try to restart the\r\n"
		    "part of the program that encountered the exception.\r\n"
			"\r\n"
		    "- Selecting \"No\" will make PureLS quit the part that\r\n"
		    "encountered the exception.\r\n"
			"\r\n"
		    "- Selecting \"Cancel\" will make PureLS ignore the\r\n"
		    "exception, which will force Windows to handle it.\r\n"
		     "\r\n"
		    "If you let Windows handle it PureLS will quit instantly.\r\n"
		    "However, there is no guarentee that this will not be the\r\n"
		    "case with whichever option you choose.  Good Luck!" )
		,_T("Exception in PureLS")
		,MB_SYSTEMMODAL|MB_ICONERROR|MB_YESNOCANCEL);

	/*--------------------------------------------------------------------------
	 If user would like Windows default processing, ok then.
	--------------------------------------------------------------------------*/
	if(IDCANCEL == nUserSelection)
	{
		return(FALSE);
	}

	DebugMessage("Getting thread starting point");

	/*--------------------------------------------------------------------------
	 Get the starting point, which will be used to process the error
	--------------------------------------------------------------------------*/
	NtQueryInformationThread = (QueryInformationThread)GetProcAddress(
		 GetModuleHandle(_T("NTDLL"))
		,"NtQueryInformationThread"
	);

    lSuccess = NtQueryInformationThread(
		 GetCurrentThread()
		,ThreadEntryProc
		,(PVOID)&ulThreadEntryProc
		,sizeof(ulThreadEntryProc)
		,0
	);

	/* If we couldn't get the starting point of the thread */
	if(0 != lSuccess || 0 == ulThreadEntryProc)
	{
		return(FALSE);
	}

	DebugMessage("Finding thread starting point in stacktrace");

	/*--------------------------------------------------------------------------
	 Find the thread starting point on the stack trace...
	--------------------------------------------------------------------------*/
	while(
		0 != pTrace
		&&
		pTrace->StackFrame.ulFunctionAddress != ulThreadEntryProc
	)
	{
		pTrace = pTrace->pOuterLevel;
	}

	/*--------------------------------------------------------------------------
		The thread starting point wasn't found in stack trace...
	--------------------------------------------------------------------------*/
	if(0 == pTrace || 0 == pTrace->pOuterLevel)
	{
		return(FALSE);
	}

	/*--------------------------------------------------------------------------
	 Setup values so that we can either restart or end the thread the way the
	 thread is supposed to end/start.
	 NOTE: Check ls_main.c for better explanation...
	--------------------------------------------------------------------------*/
	pExceptionData->ContextRecord->Ebp = \
		pTrace->pOuterLevel->StackFrame.ulFramePointer;

	/*--------------------------------------------------------------------------
	 We should try to restart the current thread.
	--------------------------------------------------------------------------*/
	if(IDYES == nUserSelection)
	{
		DebugMessage("Try restarting the thread");

		pExceptionData->ContextRecord->Esp = \
			pTrace->StackFrame.ulFramePointer + 2 * sizeof(ULONG);
		pExceptionData->ContextRecord->Eip = \
			pTrace->StackFrame.ulReturnAddress - 5;
	}

	/*--------------------------------------------------------------------------
	 We should try to end the current thread, which is done in the nicest way
	 possible, we pretend that everything worked dandy and we got back from the
	 "user callback", specifying the result in eax.
	--------------------------------------------------------------------------*/
	else
	{
		DebugMessage("Try stopping the thread nice and easy...");
		
		/* supposed to be a medium fun error constant */
		pExceptionData->ContextRecord->Eax = 0xD0BADBAD;
		pExceptionData->ContextRecord->Esp = pTrace->StackFrame.ulFramePointer;
		pExceptionData->ContextRecord->Eip = pTrace->StackFrame.ulReturnAddress;
	}

	/*--------------------------------------------------------------------------
	If we get all the way here, the exception should be fixed properly and thus
	we return TRUE to tell that we want execution to continue
	--------------------------------------------------------------------------*/
	return(TRUE);
}


/*--------PLS_ExceptionFilterProc-----------------------------------------------
 Purpose:
 - This should be called whenever an exception happens anywhere in the PureLS
   application domain. This function will forward any processing to delegates
   for those cases where it might do any good.
 - It also provides a mechanism for better exception handling. For every thread
   in the application it holds a list of PLS_ExceptionHandlers that could
   process the error in a better way.
------------------------------------------------------------------------------*/
LONG CALLBACK PLS_ExceptionFilterProc(
	EXCEPTION_POINTERS *pExceptionData
)
{
	/* TODO:
	   Add some kind of "entry-counter", so that we don't loop forever
	*/

	DebugMessage("In Exception handler");

	/*--------------------------------------------------------------------------
	 Check what type of exception occurred and act opon that.
	--------------------------------------------------------------------------*/
	switch (pExceptionData->ExceptionRecord->ExceptionCode)
	{
	/*--------------------------------------------------------------------------
	 If a really serious exception is thrown, then we won't be able to do the
	 necessary processing to correct the problem, and therefore the Windows
	 exception handler will be called upon to exit process.
	--------------------------------------------------------------------------*/
	case EXCEPTION_STACK_OVERFLOW:
	case EXCEPTION_NONCONTINUABLE_EXCEPTION:
		break;

	/*--------------------------------------------------------------------------
	 Some typical exceptions happen just because programmers tend to do errors
	 sometime. This could be assuming something about a certain value from the
	 user or working with an interface the wrong way.
	--------------------------------------------------------------------------*/
	case EXCEPTION_ACCESS_VIOLATION:
	case EXCEPTION_ARRAY_BOUNDS_EXCEEDED:
	case EXCEPTION_INVALID_HANDLE:
	case EXCEPTION_INT_DIVIDE_BY_ZERO:
		{
			assert(FALSE);
		}

		/*----------------------------------------------------------------------
		 These exceptions are considered fixable, and therefore an attemp to fix
		 the underlaying problem will be made.  First the exception will be
		 passed down to any registered exception handlers, which may be able to
		 provide a better solution to the problem, as they might know more about
		 the implementation where the problem arose.
		----------------------------------------------------------------------*/
		{
			BOOL bProcessed = FALSE;

			PCONTEXT pContext = pExceptionData->ContextRecord;

			/* Retrieve the stack trace for the exception */
			PPLS_STACKTRACE pStackTrace = PLS_StackTraceLookup(pContext);

			/* Each thread has it's own pool of Exception handlers */
			PPLS_EXCEPTIONCHAIN pChain = TlsGetValue(TLSID_ExceptionHandlers);

			DebugMessage("After getting chain of exception handlers");

			/*------------------------------------------------------------------
			 Remove all exception handlers to high on the stack, they should
			 have been deleted by "the adder", but in an exception handling
			 mechanism, one can never be to sure.

			 Enumerate all exception handlers for this thread, and invoke them
			 in the reversed order they were added.  An exception handler
			 returns FALSE has not managed to tame the exception, and therefore
			 the list will be traveled further to look for a handler that can.

			 If the handler can't process the exception, it will automaticly be
			 removed from the exception handler chain.
			------------------------------------------------------------------*/
			while(
				0 != pChain
				&&
				pContext->Esp <= pChain->StackPointer
				&&
				FALSE == ( /* assignment intended */
					bProcessed = pChain->Handler(pExceptionData, pStackTrace)
				)
			)
			{
				PPLS_EXCEPTIONCHAIN pTemp = pChain->pNext;

				PLS_ExceptionHandlerPop();
				pChain = pTemp;

				DebugMessage("Handler was no good");
			}

			/*------------------------------------------------------------------
			 No handler could protect us from the exception, therefore we must
			 try our last resort, fixing it ourselves.
			------------------------------------------------------------------*/
			if(!bProcessed)
			{
				bProcessed = PLS_ExceptionDefaultHandler(
					pExceptionData, pStackTrace
				);
			}

			/* Free stack trace, no need for it any longer */
			PLS_StackTraceFree(pStackTrace);
			pStackTrace = 0;

			/* If a handler processed the exception, try to continue */
			if(bProcessed)
			{
				DebugMessage("Someone processed the exception");
				
				return(EXCEPTION_CONTINUE_EXECUTION);
			}
		}
		break;

	/*--------------------------------------------------------------------------
	 All exceptions that are not amongst the known ones, are considered
	 unknown, and therefore we give in to the default processing.
	--------------------------------------------------------------------------*/
	default:
		break;

	}

	/*--------------------------------------------------------------------------
	 TODO: Here would could try invoking the previous filter function, if there
	       was one. The API doesn't exactly support that though, so I guess we
	       really should stay out of it.
	--------------------------------------------------------------------------*/
	DebugMessage("No handling of the specified exception was possible");

	/*--------------------------------------------------------------------------
	 Tell windows that it should execute the core exception handler.
	--------------------------------------------------------------------------*/
	return(EXCEPTION_CONTINUE_SEARCH);
}

/*--------PLS_LoadExceptionHandler----------------------------------------------
 Purpose:
 - Set up the exception handler to process exceptions inside PureLS application
   domain.
------------------------------------------------------------------------------*/
BOOL PLS_LoadExceptionHandler(
	VOID
)
{
	/* Every thread has a different set of exception handlers */
	TLSID_ExceptionHandlers = TlsAlloc();

	if (TLSID_ExceptionHandlers == TLS_OUT_OF_INDEXES)
	{
		return(FALSE);
	}

	/* Install the main exception handler procedure to begin catching */
	SetUnhandledExceptionFilter(PLS_ExceptionFilterProc);

	return(TRUE);
}

/*--------PLS_UnloadExceptionHandler--------------------------------------------
 Purpose:
 - To revert all changes made by LoadExceptionHandler.
------------------------------------------------------------------------------*/
VOID PLS_UnloadExceptionHandler(
	VOID
)
{
	/* Remove the exception catcher routine. */
	SetUnhandledExceptionFilter(0);

	/* Remove all registered exception handlers */
	while(TlsGetValue(TLSID_ExceptionHandlers))
	{
		PLS_ExceptionHandlerPop();
	}

	/* Free the thread local storage id */
	TlsFree(TLSID_ExceptionHandlers);

	return;
}

#endif /* PLS_EXCEPTION_HANDLER */

/*--------End Of File---------------------------------------------------------*/
