/*------------------------------------------------------------------------------
	This is part of the PureLS Source code.

	Copyright (C) 2000-2003 The PureLS Development Team

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the Free
    Software Foundation; either version 2 of the License, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details.

    You should have received a copy of the GNU General Public License along with
    this program; if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------
 Lets just make sure we aren't depending on IE
------------------------------------------------------------------------------*/
#define _WIN32_IE 0x0000


/*---System Includes----------------------------------------------------------*/
#include <tchar.h>
#include <windows.h>
#include <shellapi.h>
#include <objbase.h>
/*---Language Includes--------------------------------------------------------*/
#include <stdlib.h>
/*---Shell Includes-----------------------------------------------------------*/
#include "shellsvc.h"
//#include "../sdk/include/purels.h"
#define LM_SYSTRAY                     9214

/*----------------------------------------------------------------------------*/
#ifdef LOG_NOTIFYICON
#include <stdio.h>
#endif


/*---Global Variables---------------------------------------------------------*/
/* From shellsvc.c */
extern PTEPLINKLIST g_pRegistered;
extern CRITICAL_SECTION g_csCallBackList;
extern CRITICAL_SECTION g_csTrayList;
/*----------------------------------------------------------------------------*/

/*---Local Variables----------------------------------------------------------*/
static PNIDLINKLIST s_pItems;
/*----------------------------------------------------------------------------*/




/*--------SS_EnumNotifyIcons----------------------------------------------------
 Purpose:
 - Enum enumerates all current tray icons
------------------------------------------------------------------------------*/
BOOL WINAPI SS_EnumNotifyIcons(
	SHTRAYCALLBACKPROC lpCallBackFunc
)
{
	NOTIFYICONDATASH nid;
	PNIDLINKLIST p, pprev;

	if(NULL == lpCallBackFunc)
	{
		return(FALSE);
	}

	EnterCriticalSection(&g_csTrayList);

	p = s_pItems;
	pprev = NULL;

	while(NULL != p)
	{
		/* Cleanup */
		if(!IsWindow(p->hWnd))
		{
			if(NULL == pprev)
			{
				s_pItems = p->pNext;
			}
			else
			{
				pprev->pNext = p->pNext;
			}

			if(NULL != p->hIcon)
			{
				DestroyIcon(p->hIcon);
			}

			free(p);

			if(NULL == pprev)
			{
				p = s_pItems;
			}
			else
			{
				p = pprev->pNext;
			}

			continue;
		}

		memset(&nid, 0, sizeof(NOTIFYICONDATASH));

		nid.cbSize = sizeof(NOTIFYICONDATASH);
		nid.hWnd = p->hWnd;
		nid.uID = p->uID;
		nid.uFlags = p->uFlags;
		nid.uCallbackMessage = p->uMesg;
		nid.hIcon = p->hIcon;

		if(0 != *p->szTip)
		{
			_tcsncpy(nid.szTip, p->szTip, 128);
			nid.szTip[127] = 0;
		}

		if(!lpCallBackFunc(LM_SYSTRAY, NIM_ADD, (LPARAM)&nid))
		{
			break;
		}

		pprev = p;
		p = p->pNext;
	}

	LeaveCriticalSection(&g_csTrayList);

	return(NULL == p);
}


/*--------_getListItem----------------------------------------------------------
 Purpose:
 -
------------------------------------------------------------------------------*/
static PNIDLINKLIST _getListItem(
	 HWND const hWnd
	,UINT const uID
)
{
	PNIDLINKLIST p = s_pItems;

	if(NULL == p)
	{
		p = s_pItems = (PNIDLINKLIST)malloc(sizeof(NIDLINKLIST));

		if(NULL != p)
		{
			memset(p, 0, sizeof(NIDLINKLIST));

			p->hWnd = hWnd;
			p->uID = uID;
		}
	}
	else
	{
		while(NULL != p)
		{
			if(p->hWnd == hWnd && p->uID == uID)
			{
				break;
			}

			if(NULL == p->pNext)
			{
				p = p->pNext = (PNIDLINKLIST)malloc(
					sizeof(NIDLINKLIST)
				);

				if(NULL != p)
				{
					memset(p, 0, sizeof(NIDLINKLIST));

					p->hWnd = hWnd;
					p->uID = uID;
				}
				break;
			}

			p = p->pNext;
		}
	}

	return( p );
}


/*--------_findListItem---------------------------------------------------------
 Purpose:
 -
------------------------------------------------------------------------------*/
static PNIDLINKLIST _findListItem(
	 HWND const hWnd
	,UINT const uID
)
{
	PNIDLINKLIST p = s_pItems;

	while(NULL != p)
	{
		if(hWnd == p->hWnd && uID == p->uID)
		{
			break;
		}

		p = p->pNext;
	}

	return(p);
}


/*--------_delListItem----------------------------------------------------------
 Purpose:
 -
------------------------------------------------------------------------------*/
static BOOL _delListItem(
	PCNIDLINKLIST const pll
)
{
	PNIDLINKLIST p = s_pItems, pprev = NULL;

	while(NULL != p)
	{
		if(p == pll)
		{
			if(NULL == pprev)
			{
				s_pItems = p->pNext;
			}
			else
			{
				pprev->pNext = p->pNext;
			}

			if(NULL != p->hIcon)
			{
				DestroyIcon(p->hIcon);
			}

			free(p);

			return(TRUE);
		}

		pprev = p;
		p = p->pNext;
	}

	return(FALSE);
}

/*--------_copyNotifyIcon2KXP---------------------------------------------------
 Purpose:
 -
------------------------------------------------------------------------------*/
static VOID _copyNotifyIcon2KXP(
	 PNOTIFYICONDATASH const pnid
	,PCSHELLTRAYDATA const pstd
)
{
	pnid->dwState = pstd->nid.w2K.dwState;
	pnid->dwStateMask = pstd->nid.w2K.dwStateMask;

#ifndef _UNICODE
	WideCharToMultiByte(
		CP_ACP,
		0,
		pstd->nid.w2K.szInfo,
		-1,
		pnid->szInfo,
		256,
		NULL,
		NULL
	);
#else
	wcsncpy(pnid->szInfo, pstd->nid.w2K.szInfo, 256);
	pnid->szTip[255] = 0;
#endif

	pnid->DUMMYUNIONNAME.uTimeout = pstd->nid.w2K.DUMMYUNIONNAME.uTimeout;

#ifndef _UNICODE
	WideCharToMultiByte(
		CP_ACP,
		0,
		pstd->nid.w2K.szInfoTitle,
		-1,
		pnid->szInfoTitle,
		64,
		NULL,
		NULL
	);
#else
	wcsncpy(pnid->szInfoTitle, pstd->nid.w2K.szInfoTitle, 64);
	pnid->szTip[63] = 0;
#endif

	pnid->dwInfoFlags = pstd->nid.w2K.dwInfoFlags;

	if(sizeof(NOTIFYICONDATAXP) == pstd->nid.w2K.cbSize)
	{
		pnid->guidItem = pstd->nid.wXP.guidItem;
	}

	return;
}

static VOID _copyDblStrW(
	 LPTSTR const pszDest1
	,LPTSTR const pszDest2
	,LPCWSTR const pszString
	,size_t nLen
)
{
#ifndef _UNICODE
	WideCharToMultiByte(
		 CP_ACP
		,0
		,pszString
		,-1
		,pszDest1
		,nLen
		,NULL
		,NULL
	);

	strncpy(pszDest2, pszDest1, nLen);
	pszDest2[nLen-1] = 0;
#else
	wcsncpy(pszDest1, pszString, nLen);
	pszDest1[nLen-1] = 0;

	wcsncpy(pszDest2, pszDest1, nLen);
	pszDest2[nLen-1] = 0;
#endif /* _UNICODE */

	return;
}

static VOID _copyDblStrA(
	 LPTSTR const pszDest1
	,LPTSTR const pszDest2
	,LPCSTR const pszString
	,size_t nLen
)
{
#ifndef _UNICODE
	strncpy(pszDest1, pszString, nLen);
	pszDest1[nLen-1] = 0;

	strncpy(pszDest2, pszDest1, nLen);
	pszDest2[nLen-1] = 0;
#else
	MultiByteToWideChar(
		 CP_ACP
		,0
		,pszString
		,-1
		,pszDest1
		,nLen
	);

	wcsncpy(pszDest2, pszDest1, nLen);
	pszDest2[nLen-1] = 0;
#endif /* _UNICODE */

	return;
}

static PNIDLINKLIST _copyNotifyIconBase(
	 PNOTIFYICONDATASH const pnid
	,PCSHELLTRAYDATA const pstd
)
{
	PNIDLINKLIST pItem;

	/*--------------------------------------------------------------------------
	 Find item being updated/removed, or create a new item
	--------------------------------------------------------------------------*/
	pItem = _findListItem(pstd->nid.w9X.hWnd, pstd->nid.w9X.uID);

	if(NIM_ADD == pstd->dwMessage)
	{
		if(NULL == pItem)
		{
			pItem = _getListItem(pstd->nid.w9X.hWnd, pstd->nid.w9X.uID);
		}
		else /* Ignore repeat NIM_ADD messages */
		{
			pItem = NULL;
		}
	}

	if(NULL != pItem)
	{
		memset(pnid, 0, sizeof(NOTIFYICONDATASH));
		pnid->cbSize = sizeof(NOTIFYICONDATASH);
		pnid->hWnd   = pItem->hWnd;
		pnid->uID    = pItem->uID;

		pnid->uFlags = pstd->nid.w9X.uFlags;

		if(NIF_MESSAGE == (pstd->nid.w9X.uFlags & NIF_MESSAGE))
		{
			pnid->uCallbackMessage = pstd->nid.w9X.uCallbackMessage;
			pItem->uMesg = pnid->uCallbackMessage;
			pItem->uFlags |= NIF_MESSAGE;
		}

		if(NIF_ICON == (pstd->nid.w9X.uFlags & NIF_ICON))
		{
			if(NULL != pstd->nid.w9X.hIcon)
			{
				pnid->hIcon = CopyIcon(pstd->nid.w9X.hIcon);

				if(NULL == pnid->hIcon)
				{
					pnid->hIcon = (HICON)CopyImage(
						 (HANDLE)pstd->nid.w9X.hIcon
						,IMAGE_ICON
						,0
						,0
						,0
					);
				}

				if(NULL != pnid->hIcon)
				{
					if(NULL != pItem->hIcon)
					{
						DestroyIcon(pItem->hIcon);
					}

					pItem->hIcon = pnid->hIcon;
					pItem->uFlags |= NIF_ICON;
				}
			}
			else
			{
				if(NULL != pItem->hIcon)
				{
					DestroyIcon(pItem->hIcon);
					pItem->hIcon = NULL;
				}

				pItem->uFlags &= ~NIF_ICON;
			}
		}

		if(NIF_TIP == (pstd->nid.w9X.uFlags & NIF_TIP))
		{
			*pItem->szTip = *pnid->szTip = 0;

			switch(pstd->nid.w9X.cbSize)
			{
			case sizeof(NOTIFYICONDATAXP):
			case sizeof(NOTIFYICONDATA2K):
				_copyDblStrW(
					 pnid->szTip
					,pItem->szTip
					,pstd->nid.w2K.szTip
					,128
				);
				break;

			case sizeof(NOTIFYICONDATANT):
				_copyDblStrW(
					 pnid->szTip
					,pItem->szTip
					,pstd->nid.wNT.szTip
					,64
				);
				break;

			case sizeof(NOTIFYICONDATA9X):
			default:
				_copyDblStrA(
					 pnid->szTip
					,pItem->szTip
					,pstd->nid.w9X.szTip
					,64
				);
				break;
			}

			if(0 != *pItem->szTip)
			{
				pItem->uFlags |= NIF_TIP;
			}
		}
	}

	return(pItem);
}


/*--------_notifyTrayUpdate-----------------------------------------------------
 Purpose:
 - Enum all registered callback procedures and call them
------------------------------------------------------------------------------*/
static BOOL _notifyTrayUpdate(
	 PCNOTIFYICONDATASH const pnid
	,DWORD const dwMsg
)
{
	PTEPLINKLIST p;

	EnterCriticalSection(&g_csCallBackList);

	p = g_pRegistered;

#ifdef _DEBUG
	if(NULL == p)
	{
		OutputDebugString(_T("\nERROR: NOTIFYICON: No callbacks\n"));
	}
#endif

	while(NULL != p)
	{
		if(!p->pProc(LM_SYSTRAY, (WPARAM)dwMsg, (LPARAM)pnid))
		{
			break;
		}

		p = p->pNext;
	}

	LeaveCriticalSection(&g_csCallBackList);

	return(NULL == p);
}



/*------------------------------------------------------------------------------
------------------------------------------------------------------------------*/
#ifdef LOG_NOTIFYICON
extern FILE * fNotifyLog;

static void _logNotifyIcon(PCSHELLTRAYDATA const pstd)
{
	_TCHAR szBuffer[1024];

	if(NULL == fNotifyLog)
	{
		return;
	}

	_ftprintf(fNotifyLog, _T("=============\n"));
	_ftprintf(fNotifyLog, _T("=NotifyIcon:=\n"));
	_ftprintf(fNotifyLog, _T("================================================================================\n"));

	*szBuffer = 0;
	_ftprintf(fNotifyLog, _T("(0x%08X) Title: %s"), pstd->nid.w9X.hWnd, szBuffer);

	*szBuffer = 0;
	GetClassName(pstd->nid.w9X.hWnd, szBuffer, 1024);
	_ftprintf(fNotifyLog, _T(" - Class: %s\n\n"), szBuffer);

	_ftprintf(fNotifyLog, _T("dwMessage:"));
	switch(pstd->dwMessage)
	{
	case NIM_ADD:
		_ftprintf(fNotifyLog, _T(" NIM_ADD"));
		break;
	case NIM_MODIFY:
		_ftprintf(fNotifyLog, _T(" NIM_MODIFY"));
		break;
	case NIM_DELETE:
		_ftprintf(fNotifyLog, _T(" NIM_DELETE"));
		break;
	case NIM_SETFOCUS:
		_ftprintf(fNotifyLog, _T(" NIM_SETFOCUS"));
		break;
	case NIM_SETVERSION:
		_ftprintf(fNotifyLog, _T(" NIM_SETVERSION"));
		break;
	}
	_ftprintf(fNotifyLog, _T("\n"));


	_ftprintf(fNotifyLog, _T("   uFlags:"));
	if(NIF_MESSAGE == (NIF_MESSAGE & pstd->nid.w9X.uFlags))
	{
		_ftprintf(fNotifyLog, _T(" NIF_MESSAGE"));
	}
	if(NIF_ICON == (NIF_ICON & pstd->nid.w9X.uFlags))
	{
		_ftprintf(fNotifyLog, _T(" NIF_ICON"));
	}
	if(NIF_TIP == (NIF_TIP & pstd->nid.w9X.uFlags))
	{
		_ftprintf(fNotifyLog, _T(" NIF_TIP"));
	}
	if(NIF_STATE == (NIF_STATE & pstd->nid.w9X.uFlags))
	{
		_ftprintf(fNotifyLog, _T(" NIF_STATE"));
	}
	if(NIF_INFO == (NIF_INFO & pstd->nid.w9X.uFlags))
	{
		_ftprintf(fNotifyLog, _T(" NIF_INFO"));
	}
	if(NIF_GUID == (NIF_GUID & pstd->nid.w9X.uFlags))
	{
		_ftprintf(fNotifyLog, _T(" NIF_GUID"));
	}
	_ftprintf(fNotifyLog, _T("\n"));


	if(
		(pstd->nid.w9X.cbSize >= sizeof(NOTIFYICONDATA2K))
		&&
		(NIF_STATE == (NIF_STATE & pstd->nid.w2K.uFlags))
	)
	{
		_ftprintf(fNotifyLog, _T("  dwState:"));
		if(
			(pstd->nid.w2K.dwStateMask & NIS_HIDDEN)
			&&
			(pstd->nid.w2K.dwState & NIS_HIDDEN)
		)
		{
			_ftprintf(fNotifyLog, _T(" NIS_HIDDEN"));
		}
		if(
			(pstd->nid.w2K.dwStateMask & NIS_SHAREDICON)
			&&
			(pstd->nid.w2K.dwState & NIS_SHAREDICON)
		)
		{
			_ftprintf(fNotifyLog, _T(" NIS_SHAREDICON"));
		}
		_ftprintf(fNotifyLog, _T("\n"));
	}

	EnterCriticalSection(&g_csTrayList);
	if(NULL == _findListItem(pstd->nid.w9X.hWnd, pstd->nid.w9X.uID))
	{
		if(NIM_ADD != pstd->dwMessage)
		{
			_ftprintf(fNotifyLog, _T("ERROR: Icon does not exist!\n"));
		}
	}
	LeaveCriticalSection(&g_csTrayList);
	if(NIF_ICON == (NIF_ICON & pstd->nid.w9X.uFlags) && NULL == pstd->nid.w9X.hIcon)
	{
		_ftprintf(fNotifyLog, _T("ERROR: NIF_ICON while hIcon is NULL\n"));
	}
	if(NIF_MESSAGE == (NIF_MESSAGE & pstd->nid.w9X.uFlags) && 0 == pstd->nid.w9X.uCallbackMessage)
	{
		_ftprintf(fNotifyLog, _T("ERROR: NIF_MESSAGE while uCallbackMessage is zero\n"));
	}

	_ftprintf(fNotifyLog, _T("================================================================================\n"));
	_ftprintf(fNotifyLog, _T("SHELLTRAYDATA\n{\n"));
	_ftprintf(fNotifyLog, _T("\tDWORD dwUnknown = 0x%08X\n"), pstd->dwUnknown);
	_ftprintf(fNotifyLog, _T("\tDWORD dwMessage = 0x%08X\n"), pstd->dwMessage);

	switch(pstd->nid.w9X.cbSize)
	{
	case sizeof(NOTIFYICONDATAXP):
		_ftprintf(fNotifyLog, _T("\tNOTIFYICONDATAXP\n"));
		break;
	case sizeof(NOTIFYICONDATA2K):
		_ftprintf(fNotifyLog, _T("\tNOTIFYICONDATA2K\n"));
		break;
	case sizeof(NOTIFYICONDATANT):
		_ftprintf(fNotifyLog, _T("\tNOTIFYICONDATANT\n"));
		break;
	case sizeof(NOTIFYICONDATA9X):
		_ftprintf(fNotifyLog, _T("\tNOTIFYICONDATA9X\n"));
		break;
	default:
		_ftprintf(fNotifyLog, _T("\tUNKNOWN_NID\n"));
		break;
	}

	_ftprintf(fNotifyLog, _T("\t{\n"));
	_ftprintf(fNotifyLog, _T("\t\tDWORD cbSize           = 0x%08X\n"), pstd->nid.w9X.cbSize);
	_ftprintf(fNotifyLog, _T("\t\tHWND  hWnd             = 0x%08X\n"), pstd->nid.w9X.hWnd);
	_ftprintf(fNotifyLog, _T("\t\tUINT  uID              = 0x%08X\n"), pstd->nid.w9X.uID);
	_ftprintf(fNotifyLog, _T("\t\tUINT  uFlags           = 0x%08X\n"), pstd->nid.w9X.uFlags);
	_ftprintf(fNotifyLog, _T("\t\tUINT  uCallbackMessage = 0x%08X\n"), pstd->nid.w9X.uCallbackMessage);
	_ftprintf(fNotifyLog, _T("\t\tHICON hIcon            = 0x%08X\n"), pstd->nid.w9X.hIcon);

	switch(pstd->nid.w9X.cbSize)
	{
	case sizeof(NOTIFYICONDATAXP):
		if(NIF_TIP == (pstd->nid.wXP.uFlags & NIF_TIP))
		{
#		ifdef UNICODE
			_tcsncpy(szBuffer, pstd->nid.wXP.szTip, 128);
#		else
			WideCharToMultiByte(
				CP_ACP,
				0,
				pstd->nid.wXP.szTip,
				-1,
				szBuffer,
				128 * sizeof(_TCHAR),
				NULL,
				NULL
			);
#		endif
		}
		else
		{
			*szBuffer = 0;
		}

	_ftprintf(fNotifyLog, _T("\t\tWCHAR  szTip[128]      = %s\n"), szBuffer);
		break;

	case sizeof(NOTIFYICONDATA2K):
		if(NIF_TIP == (pstd->nid.w2K.uFlags & NIF_TIP))
		{
#		ifdef UNICODE
			_tcsncpy(szBuffer, pstd->nid.w2K.szTip, 128);
#		else
			WideCharToMultiByte(
				CP_ACP,
				0,
				pstd->nid.w2K.szTip,
				-1,
				szBuffer,
				128 * sizeof(_TCHAR),
				NULL,
				NULL
			);
#		endif
		}
		else
		{
			*szBuffer = 0;
		}

	_ftprintf(fNotifyLog, _T("\t\tWCHAR szTip[128]       = %s\n"), szBuffer);
		break;

	case sizeof(NOTIFYICONDATANT):
		if(NIF_TIP == (pstd->nid.wNT.uFlags & NIF_TIP))
		{
#		ifdef UNICODE
			_tcsncpy(szBuffer, pstd->nid.wNT.szTip, 64);
#		else
			WideCharToMultiByte(
				CP_ACP,
				0,
				pstd->nid.wNT.szTip,
				-1,
				szBuffer,
				64 * sizeof(_TCHAR),
				NULL,
				NULL
			);
#		endif
		}
		else
		{
			*szBuffer = 0;
		}

	_ftprintf(fNotifyLog, _T("\t\tWCHAR szTip[64]        = %s\n"), szBuffer);
		break;

	case sizeof(NOTIFYICONDATA9X):
#ifndef _UNICODE
		strncpy(szBuffer, pstd->nid.w9X.szTip, 64);
		szBuffer[63] = 0;
#else
		MultiByteToWideChar(
			 CP_ACP
			,0
			,pstd->nid.w9X.szTip
			,-1
			,szBuffer
			,64
		);
#endif /* _UNICODE */
	_ftprintf(fNotifyLog, _T("\t\tCHAR  szTip[64]        = %s\n"), szBuffer);
		break;
	default:
		_ftprintf(fNotifyLog, _T("\t\tUNKNOWN_NID\n"));
		break;
	}

	switch(pstd->nid.w9X.cbSize)
	{
	case sizeof(NOTIFYICONDATAXP):
	case sizeof(NOTIFYICONDATA2K):
	_ftprintf(fNotifyLog, _T("\t\tDWORD dwState          = 0x%08X\n"), pstd->nid.w2K.dwState);
	_ftprintf(fNotifyLog, _T("\t\tDWORD dwStateMask      = 0x%08X\n"), pstd->nid.w2K.dwStateMask);

		if(NIF_INFO == (pstd->nid.wNT.uFlags & NIF_INFO))
		{
#		ifdef UNICODE
			_tcsncpy(szBuffer, pstd->nid.w2K.szInfo, 256);
#		else
			WideCharToMultiByte(
				CP_ACP,
				0,
				pstd->nid.w2K.szInfo,
				-1,
				szBuffer,
				256 * sizeof(_TCHAR),
				NULL,
				NULL
			);
#		endif
		}
		else
		{
			*szBuffer = 0;
		}
	_ftprintf(fNotifyLog, _T("\t\tWCHAR szInfo[256]      = %s\n"), szBuffer);

	_ftprintf(fNotifyLog, _T("\t\tunion\n\t\t{\n"));
	_ftprintf(fNotifyLog, _T("\t\t\tUINT uTimeout      = 0x%08X\n"), pstd->nid.w2K.u.uTimeout);
	_ftprintf(fNotifyLog, _T("\t\t\tUINT uVersion      = 0x%08X\n"), pstd->nid.w2K.u.uVersion);
	_ftprintf(fNotifyLog, _T("\t\t}\n"));

		if(NIF_INFO == (pstd->nid.w2K.uFlags & NIF_INFO))
		{
#		ifdef UNICODE
			_tcsncpy(szBuffer, pstd->nid.w2K.szInfoTitle, 64);
#		else
			WideCharToMultiByte(
				CP_ACP,
				0,
				pstd->nid.w2K.szInfoTitle,
				-1,
				szBuffer,
				64 * sizeof(_TCHAR),
				NULL,
				NULL
			);
#		endif
		}
		else
		{
			*szBuffer = 0;
		}
	_ftprintf(fNotifyLog, _T("\t\tWCHAR szInfoTitle[64]  = %s\n"), szBuffer);
	_ftprintf(fNotifyLog, _T("\t\tDWORD dwInfoFlags      = 0x%08X\n"), pstd->nid.w2K.dwInfoFlags);
		break;

	case sizeof(NOTIFYICONDATANT):
		break;
	case sizeof(NOTIFYICONDATA9X):
		break;
	default:
		break;
	}

	if(sizeof(NOTIFYICONDATAXP) == pstd->nid.w9X.cbSize)
	{
		LPWSTR   pwsz;
		_TCHAR   szCLSID[MAX_PATH];
		LPMALLOC pMalloc;

		// Get the CLSID in string form.
		StringFromIID(&pstd->nid.wXP.guidItem, &pwsz);

		if(pwsz)
		{
#		ifdef UNICODE
			_tcsncpy(szCLSID, pwsz, MAX_PATH);
#		else
			WideCharToMultiByte(
				CP_ACP,
				0,
				pwsz,
				-1,
				szCLSID,
				MAX_PATH * sizeof(TCHAR),
				NULL,
				NULL
			);
#		endif

			// Free the string.
			CoGetMalloc(1, &pMalloc);
			pMalloc->lpVtbl->Free(pMalloc, pwsz);
			pMalloc->lpVtbl->Release(pMalloc);

			szCLSID[MAX_PATH-1] = 0;

	_ftprintf(fNotifyLog, _T("\t\tGUID guidItem          = %s\n"), szCLSID);
		}
	}

	_ftprintf(fNotifyLog, _T("\t}\n}\n"));
	_ftprintf(fNotifyLog, _T("================================================================================\n\n"));

	fflush(fNotifyLog);

	return;
}
#endif /* LOG_NOTIFYICON */


/*--------onTrayNotify----------------------------------------------------------
 Purpose:
 -
------------------------------------------------------------------------------*/
LRESULT onNotifyIcon( PCSHELLTRAYDATA const pstd )
{
	NOTIFYICONDATASH nid;
	PNIDLINKLIST pItem = NULL;
	LRESULT lResult = 0;

#ifdef LOG_NOTIFYICON
	_logNotifyIcon(pstd);
#endif

//	if(!IsWindow(pstd->nid.w9X.hWnd))
//	{
//		pstd->dwMessage = NIM_DELETE;
//	}

	EnterCriticalSection(&g_csTrayList);

	switch(pstd->nid.w9X.cbSize)
	{
	case sizeof(NOTIFYICONDATAXP):
	case sizeof(NOTIFYICONDATA2K):
		/* Handle special cases for hidden/shared icons */
		if((NIF_ICON|NIF_STATE) == (pstd->nid.w2K.uFlags&(NIF_ICON|NIF_STATE)))
		{
			/* We currently do not support hidden icons */
			if(IS2KXPHIDDENICON(pstd->nid.w2K))
			{
				/*--------------------------------------------------------------
				TEMPORARY HACK:  Because we do not store hidden icons currently,
				then NIM_DELETE request would fail if we processed it normally
				because the icon would not be found.  So, it is better to return
				success for every hidden icon deletion request.  Otherwise, if
				it is a NIM_ADD/MODIFY, only return success if hIcon is valid.
				I plan on re-adding hidden icon support in the future, and
				at that point this hack will not be necessary.
				--------------------------------------------------------------*/
				if(NIM_DELETE == pstd->dwMessage || NULL != pstd->nid.w2K.hIcon)
				{
					lResult = 1;
				}
				break;
			}

			/*------------------------------------------------------------------
			 We currently do not implement shared icons, treat them as normal
			 icons, except for the following check.
			------------------------------------------------------------------*/
			if(IS2KXPSHAREDICON(pstd->nid.w2K))
			{
				/*--------------------------------------------------------------
				 We must not allow shared icons to be added if they do not have
				 a valid hIcon.  So, unless it is a delete request, return false
				 if hIcon is invalid.
				--------------------------------------------------------------*/
				if(NIM_DELETE != pstd->dwMessage && NULL == pstd->nid.w2K.hIcon)
				{
					break;
				}
			}
		}

		pItem = _copyNotifyIconBase(&nid, pstd);

		if(NULL != pItem)
		{
			_copyNotifyIcon2KXP(&nid, pstd);
		}
		break;

	default:
		pItem = _copyNotifyIconBase(&nid, pstd);
		break;
	}

	LeaveCriticalSection(&g_csTrayList);

	if(
		NULL == pItem
		&&
		(
			NIM_SETFOCUS == pstd->dwMessage
			||
			NIM_SETVERSION == pstd->dwMessage
		)
	)
	{
		lResult = 1;
	}
	else if(NULL != pItem)
	{
		/* Notify all Callbacks of the change */
		_notifyTrayUpdate(&nid, pstd->dwMessage);

		/* Remove Item from our list if it was deleted */
		if(NIM_DELETE == pstd->dwMessage)
		{
			EnterCriticalSection(&g_csTrayList);
			_delListItem(pItem);
			LeaveCriticalSection(&g_csTrayList);
		}

		lResult = 1;
	}

	/* return success/failure */
	return(lResult);
}


/*--------freeNotifyList--------------------------------------------------------
 Purpose:
 -
------------------------------------------------------------------------------*/
VOID freeNotifyList(
	VOID
)
{
	PNIDLINKLIST p;

	EnterCriticalSection(&g_csTrayList);

	while(NULL != s_pItems)
	{
		p = s_pItems->pNext;

		if(NULL != s_pItems->hIcon)
		{
			DestroyIcon(s_pItems->hIcon);
		}

		free(s_pItems);

		s_pItems = p;
	}

	LeaveCriticalSection(&g_csTrayList);

	return;
}

/*--------End Of File---------------------------------------------------------*/
