/*------------------------------------------------------------------------------
	This is part of the PureLS Source code.

	Copyright (C) 2000-2003 The PureLS Development Team

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the Free
    Software Foundation; either version 2 of the License, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details.

    You should have received a copy of the GNU General Public License along with
    this program; if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------
 Lets just make sure we aren't depending on IE
------------------------------------------------------------------------------*/
#define _WIN32_IE 0x0000


/*---System Includes----------------------------------------------------------*/
#include <tchar.h>
#include <windows.h>
#include <objbase.h>
#include <shellapi.h>
/*---Language Includes--------------------------------------------------------*/
#include <stdlib.h>
/*---Shell Includes-----------------------------------------------------------*/
#include "shellsvc.h"
//#include "../sdk/include/purels.h"
#include "../sdk/include/plsdefs.h"
/*----------------------------------------------------------------------------*/

#if defined(LOG_APPBAR) || defined(LOG_NOTIFYICON) || defined(LOG_WM_COPYDATA)
#include <stdio.h>
#endif

#ifdef SHELLSVC_APPBAR_ENABLE
/* appbar.c */
extern VOID freeAppBarList( VOID );
extern LRESULT onAppbarMsg( LPVOID, DWORD const );
#endif /* SHELLSVC_APPBAR_ENABLE */

/* notifyicon.c */
extern VOID freeNotifyList( VOID );
extern LRESULT onNotifyIcon( PCSHELLTRAYDATA const );

/* ssobjects.c */
extern VOID loadShellServiceObjects( VOID );
extern VOID unloadShellServiceObjects( VOID );

/* shellhook.c */
extern VOID initShellHook( HWND );
extern VOID deinitShellHook( HWND );
extern BOOL onShellHook( HWND, UINT, WPARAM, LPARAM );

/*---Global Variables---------------------------------------------------------*/
HWND s_hMainWnd;

static HINSTANCE s_hMainInst;
static HWND s_hTrayStubWnd;
static ATOM s_TrayStubAtom;

PTEPLINKLIST g_pRegistered;
CRITICAL_SECTION g_csCallBackList;
CRITICAL_SECTION g_csTrayList;
CRITICAL_SECTION g_csAppBarList;
/*----------------------------------------------------------------------------*/


#ifdef LOG_APPBAR
FILE * fAppBarLog;
#endif
#ifdef LOG_NOTIFYICON
FILE * fNotifyLog;
#endif
#ifdef LOG_WM_COPYDATA
FILE * fLog;
#endif



/*--------SS_RegisterCallback---------------------------------------------------
 Purpose:
 - Registers the callback for notifications of new tray events
------------------------------------------------------------------------------*/
BOOL WINAPI SS_RegisterCallback(
	SHTRAYCALLBACKPROC lpCallbackFunc
)
{
	if(NULL != s_hMainWnd)
	{
		/* create new entry */
		PTEPLINKLIST p = (PTEPLINKLIST)malloc(sizeof(TEPLINKLIST));

		if( NULL != p )
		{
			p->pProc = lpCallbackFunc;

			EnterCriticalSection(&g_csCallBackList);

			/* set new entry */
			p->pNext = g_pRegistered;
			g_pRegistered = p;

			LeaveCriticalSection(&g_csCallBackList);

			return( TRUE );
		}
	}

	return( FALSE );
}


/*--------SS_UnregisterCallback-------------------------------------------------
 Purpose:
 - Register, registers the function for notifications of new tray events
------------------------------------------------------------------------------*/
BOOL WINAPI SS_UnregisterCallback(
	SHTRAYCALLBACKPROC lpCallbackFunc
)
{
	PTEPLINKLIST p, pprev = NULL;

	EnterCriticalSection(&g_csCallBackList);

	p = g_pRegistered;

	while( NULL != p )
	{
		if( lpCallbackFunc == p->pProc )
		{
			if( NULL == pprev )
			{
				g_pRegistered = p->pNext;
			}
			else
			{
				pprev->pNext = p->pNext;
			}

			break;
		}

		pprev = p;
		p = p->pNext;
	}

	LeaveCriticalSection(&g_csCallBackList);

	if( NULL != p )
	{
		free(p);
		return( TRUE );
	}

	return( FALSE );
}



/*--------_freeCallbacks--------------------------------------------------------
 Purpose:
 -
------------------------------------------------------------------------------*/
static VOID _freeCallbacks(
	VOID
)
{
	PTEPLINKLIST p;

	EnterCriticalSection(&g_csCallBackList);

	while( NULL != g_pRegistered )
	{
		p = g_pRegistered->pNext;

		free(g_pRegistered);

		g_pRegistered = p;
	}

	LeaveCriticalSection(&g_csCallBackList);

	return;
}

#if defined(LOG_WM_COPYDATA) || defined(LOG_NOTIFYICON) || defined(LOG_APPBAR)
static VOID _getPath(LPTSTR const pszAppPath, size_t const tLen)
{
	DWORD len;

	len = GetModuleFileName(NULL, pszAppPath, tLen);
	while(len && pszAppPath[--len] != _T('\\'));
	pszAppPath[len] = 0;

	return;
}
#endif


#ifdef LOG_WM_COPYDATA
static VOID _logWMCopyData(const PCOPYDATASTRUCT pcds)
{
	if(NULL != fLog)
	{
		UINT nIndex = 0;

		switch(pcds->dwData)
		{
		case SH_APPBAR_MSG_DATA:
			_ftprintf(fLog, _T("WM_COPYDATA(SH_APPBAR_MSG_DATA): "));
			break;

		case SH_LOAD_PROC_DATA:
			_ftprintf(fLog, _T("WM_COPYDATA(SH_LOAD_PROC_DATA): "));
			break;

		default:
			_ftprintf(fLog, _T("WM_COPYDATA(unknown): "));
			break;
		}

		_ftprintf(
			 fLog
			,_T("%d bytes at 0x%08X\r\n\t")
			,pcds->cbData
			,pcds->lpData
		);

		if(SH_LOAD_PROC_DATA == pcds->dwData)
		{
			LPWSTR   pwsz;
			REFCLSID rclsid = (REFCLSID)pcds->lpData;
			_TCHAR   szCLSID[MAX_PATH];
			LPMALLOC pMalloc;

			// Get the CLSID in string form.
			StringFromIID(rclsid, &pwsz);

			if(pwsz)
			{
#			ifdef UNICODE
				_tcsncpy(szCLSID, pwsz, MAX_PATH);
#			else
				WideCharToMultiByte(
					CP_ACP,
					0,
					pwsz,
					-1,
					szCLSID,
					MAX_PATH * sizeof(TCHAR),
					NULL,
					NULL
				);
#			endif

				// Free the string.
				CoGetMalloc(1, &pMalloc);
				pMalloc->lpVtbl->Free(pMalloc, pwsz);
				pMalloc->lpVtbl->Release(pMalloc);

				szCLSID[MAX_PATH-1] = 0;

				_ftprintf(fLog, _T("%s\r\n\t"), szCLSID);
			}
		}

		while(nIndex != pcds->cbData)
		{
			_ftprintf(fLog, _T("0x%02X "), *((BYTE*)pcds->lpData + nIndex));
			nIndex++;
		}

		_ftprintf(fLog, _T("\r\n\r\n"));
	}

	fflush(fLog);

	return;
}
#endif /* LOG_WM_COPYDATA */

#ifdef SHELLSVC_LOADINPROC_ENABLE

extern VOID addAndExecuteSSO(LPCTSTR, DWORD);

LRESULT onLoadProc( PVOID lpData, DWORD const cbData )
{
	LPWSTR   pwsz;
	REFCLSID rclsid = (REFCLSID)lpData;
	_TCHAR   szCLSID[MAX_PATH];
	LPMALLOC pMalloc;

	// Get the CLSID in string form.
	StringFromIID(rclsid, &pwsz);

	if(pwsz)
	{
#	ifdef UNICODE
		_tcsncpy(szCLSID, pwsz, MAX_PATH);
#	else
		WideCharToMultiByte(
			CP_ACP,
			0,
			pwsz,
			-1,
			szCLSID,
			MAX_PATH * sizeof(TCHAR),
			NULL,
			NULL
		);
#	endif

		// Free the string.
		CoGetMalloc(1, &pMalloc);
		pMalloc->lpVtbl->Free(pMalloc, pwsz);
		pMalloc->lpVtbl->Release(pMalloc);

		szCLSID[MAX_PATH-1] = 0;

		addAndExecuteSSO(szCLSID, _tcslen(szCLSID)*sizeof(_TCHAR));
	}

	return(0);
}
#endif /* SHELLSVC_LOADINPROC_ENABLE */

/*--------ShellWndProc----------------------------------------------------------
 Purpose:
 -
------------------------------------------------------------------------------*/
LRESULT ShellWndProc(
	 HWND hWnd
	,UINT uMsg
	,WPARAM wParam
	,LPARAM lParam
)
{
	switch( uMsg )
	{
	case WM_DESTROY:
		/* free resources allocated from WM_COPYDATA */
		freeNotifyList();
#	ifdef SHELLSVC_APPBAR_ENABLE
		freeAppBarList();
#	endif /* SHELLSVC_APPBAR_ENABLE */

		s_hMainWnd = NULL;
		PostQuitMessage(0);
		break;

	case WM_COPYDATA:
		{
			PCOPYDATASTRUCT pcds = (PCOPYDATASTRUCT)lParam;

			if(NULL != pcds && NULL != pcds->lpData)
			{
				switch(pcds->dwData)
				{
				case SH_APPBAR_MSG_DATA:
#				ifdef SHELLSVC_APPBAR_ENABLE
					return(onAppbarMsg(pcds->lpData, pcds->cbData));
#				else
#				ifdef LOG_WM_COPYDATA
					_logWMCopyData(pcds);
					break;
#				endif /* LOG_WM_COPYDATA */
#				endif /* SHELLSVC_APPBAR_ENABLE */

				case SH_NOTIFY_ICON_DATA:
					return(onNotifyIcon((PSHELLTRAYDATA)pcds->lpData));

				case SH_LOAD_PROC_DATA:
#				ifdef SHELLSVC_LOADINPROC_ENABLE
					return(onLoadProc(pcds->lpData, pcds->cbData));
#				else
#				ifdef LOG_WM_COPYDATA
					_logWMCopyData(pcds);
					break;
#				endif /* LOG_WM_COPYDATA */
#				endif /* SH_LOAD_PROC_DATA */

				default:
#				ifdef LOG_WM_COPYDATA
					_logWMCopyData(pcds);
#				endif
					break;
				}
			}
		}
		break;

	default:
		if(!onShellHook(hWnd, uMsg, wParam, lParam))
		{
			return(DefWindowProc(hWnd, uMsg, wParam, lParam));
		}
		break;
	}

	return(0);
}

LRESULT TrayStubProc(
	 HWND hWnd
	,UINT uMsg
	,WPARAM wParam
	,LPARAM lParam
)
{
	if(SS_NOTIFYTRAY == uMsg)
	{
		switch(wParam)
		{
		case NT_SETTRAYHWND:
			break;

		case NT_SETTRAYMOVE:
			SetWindowPos(
				 hWnd
				,HWND_NOTOPMOST
				,LOWORD(lParam)
				,HIWORD(lParam)
				,0
				,0
				,SWP_NOSENDCHANGING|SWP_NOZORDER|SWP_NOACTIVATE
				 |
				 SWP_NOOWNERZORDER|SWP_NOREDRAW|SWP_NOSIZE
			);
			break;

		case NT_SETTRAYSIZE:
			SetWindowPos(
				 hWnd
				,HWND_NOTOPMOST
				,0
				,0
				,LOWORD(lParam)
				,HIWORD(lParam)
				,SWP_NOSENDCHANGING|SWP_NOZORDER|SWP_NOACTIVATE
				 |
				 SWP_NOOWNERZORDER|SWP_NOREDRAW|SWP_NOMOVE
			);
			break;
		}

		return(0);
	}

	return(DefWindowProc(hWnd, uMsg, wParam, lParam));
}

VOID createTrayStub( VOID )
{
	WNDCLASS wc;

	/* Create our class */
	memset(&wc, 0, sizeof(WNDCLASS));
	wc.lpfnWndProc = (WNDPROC)TrayStubProc;
	wc.hInstance = s_hMainInst;
	wc.lpszClassName = _T("TrayNotifyWnd");

	/* Register our class */
	s_TrayStubAtom = RegisterClass(&wc);

	if(0 != s_TrayStubAtom)
	{
		/* Create our window */
		s_hTrayStubWnd = CreateWindowEx(
			 WS_EX_TOOLWINDOW
			,MAKEINTATOM(s_TrayStubAtom)
			,NULL
			,WS_CHILD
			,0,0,0,0
			,s_hMainWnd
			,(HMENU)NULL
			,s_hMainInst
			,NULL
		);
	}

	return;
}

VOID deleteTrayStub( VOID )
{
	if(NULL != s_hTrayStubWnd)
	{
		DestroyWindow(s_hTrayStubWnd);
		s_hTrayStubWnd = NULL;
	}

	if(0 != s_TrayStubAtom)
	{
		UnregisterClass(MAKEINTATOM(s_TrayStubAtom), s_hMainInst);
		s_TrayStubAtom = 0;
	}

	return;
}

/*--------SS_Main---------------------------------------------------------------
 Purpose:
 -
------------------------------------------------------------------------------*/
INT WINAPI SS_Main(
	 BOOL bNT5Tray
	,HANDLE hReadyEvent
)
{
	WNDCLASS wc;
	ATOM aClassAtom;
	INT nChkMsgErr;
	MSG sMsg;

#if defined(LOG_NOTIFYICON) || defined(LOG_WM_COPYDATA) || defined(LOG_APPBAR)
	_TCHAR szPath[MAX_PATH];
#endif

	if(NULL != FindWindow(TEXT__SHELLTRAY_CLASS, NULL))
	{
		return(-1);
	}

#ifdef LOG_APPBAR
	_getPath(szPath, MAX_PATH-16);
	_tcscat(szPath, _T("\\appbar.log"));

	fAppBarLog = _tfopen(szPath, _T("w"));
#endif

#ifdef LOG_NOTIFYICON
	_getPath(szPath, MAX_PATH-16);
	_tcscat(szPath, _T("\\notifyicon.log"));

	fNotifyLog = _tfopen(szPath, _T("w"));
#endif

#ifdef LOG_WM_COPYDATA
	_getPath(szPath, MAX_PATH-16);
	_tcscat(szPath, _T("\\wmcopydata.log"));

	fLog = _tfopen(szPath, _T("w"));
#endif

	sMsg.wParam = (WPARAM)-1;

	/* Create our class */
	memset(&wc, 0, sizeof(WNDCLASS));
	wc.style = CS_GLOBALCLASS;
	wc.lpfnWndProc = (WNDPROC)ShellWndProc;
	wc.hInstance = s_hMainInst;
	wc.lpszClassName = TEXT__SHELLTRAY_CLASS;

	/* Register our class */
	aClassAtom = RegisterClass(&wc);

	if(0 != aClassAtom)
	{
		/* Create our window */
		s_hMainWnd = CreateWindowEx(
			 WS_EX_TOPMOST|WS_EX_TOOLWINDOW
			,MAKEINTATOM(aClassAtom)
			,NULL
			,WS_POPUP
			,0,0,0,0
			,(HWND)NULL
			,(HMENU)NULL
			,s_hMainInst
			,NULL
		);

		if(NULL != s_hMainWnd)
		{
			/* Configure the system's shell hook. */
			initShellHook(s_hMainWnd);

			createTrayStub();

			if(NULL != hReadyEvent)
			{
				SetEvent(hReadyEvent);
			}

			/* Post global message indicating tray service is ready */
			PostMessage(
				 HWND_BROADCAST
				,RegisterWindowMessage(_T("TaskbarCreated"))
				,0
				,0
			);

			if( bNT5Tray )
			{
				loadShellServiceObjects();
			}

			/* Process Messages */
			while( 0 != (nChkMsgErr = GetMessage(&sMsg, NULL, 0, 0)) )
			{
				if( -1 == nChkMsgErr )
				{
#				ifdef _DEBUG
					_TCHAR tmp[128];
					_stprintf(
						 tmp
						,_T("GetMessage failed: 0x%08X")
						,GetLastError()
					);
					OutputDebugString(tmp);
#				endif
				}
				else if( NULL == sMsg.hwnd )
				{
					if( PM_QUIT == sMsg.message )
					{
						/* Clear the system shell hook */
						deinitShellHook(s_hMainWnd);
						/* Unload shell objects */
						unloadShellServiceObjects();
						/* Delete our tray stub */
						deleteTrayStub();
						/* Destroy our window */
						DestroyWindow(s_hMainWnd);
					}
				}
				else
				{
					DispatchMessage(&sMsg);
				}
			}

			/* Shouldn't be necessary */
			if(NULL != s_hMainWnd)
			{
				deinitShellHook(s_hMainWnd);
				unloadShellServiceObjects();
				deleteTrayStub();
				DestroyWindow(s_hMainWnd);
			}

			_freeCallbacks();
		}

		/* unregister window class */
		UnregisterClass(MAKEINTATOM(aClassAtom), s_hMainInst);
	}

#ifdef LOG_WM_COPYDATA
	if(NULL != fLog)
	{
		fclose(fLog);
		fLog = NULL;
	}
#endif
#ifdef LOG_NOTIFYICON
	if(NULL != fNotifyLog)
	{
		fclose(fNotifyLog);
		fNotifyLog = NULL;
	}
#endif
#ifdef LOG_APPBAR
	if(NULL != fAppBarLog)
	{
		fclose(fAppBarLog);
		fAppBarLog = NULL;
	}
#endif

	return( sMsg.wParam );
}


/*--------DllMain---------------------------------------------------------------
 Purpose:
 -
------------------------------------------------------------------------------*/
BOOL WINAPI DllMain(
	HINSTANCE hinstDLL,
	DWORD fdwReason,
	LPVOID lpvReserved
)
{
	BOOL retVal = TRUE;

    switch( fdwReason )
    {
    case DLL_PROCESS_ATTACH:
		if( DisableThreadLibraryCalls(hinstDLL) )
		{
			s_hMainInst = hinstDLL;
			InitializeCriticalSection(&g_csAppBarList);
			InitializeCriticalSection(&g_csTrayList);
			InitializeCriticalSection(&g_csCallBackList);
		}
		else
		{
			retVal = FALSE;
		}
		break;

	case DLL_PROCESS_DETACH:
		DeleteCriticalSection(&g_csCallBackList);
		DeleteCriticalSection(&g_csTrayList);
		DeleteCriticalSection(&g_csAppBarList);
		s_hMainInst = NULL;
		break;
	}

	return( retVal );
	UNREFERENCED_PARAMETER(lpvReserved);
}

/*--------End Of File---------------------------------------------------------*/
