/*------------------------------------------------------------------------------
	This is part of the PureLS Source code.

	Copyright (C) 2000-2003 The PureLS Development Team

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the Free
    Software Foundation; either version 2 of the License, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details.

    You should have received a copy of the GNU General Public License along with
    this program; if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


/*---System Includes----------------------------------------------------------*/
#include <tchar.h>
#include <windows.h>
#include <shlobj.h>
#include <regstr.h>
/*----------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------
 MinGW lacks a couple definitions for Shell Object support
------------------------------------------------------------------------------*/
#ifdef __MINGW32__
#include <initguid.h>

/* CGID_ShellServiceObject:		  000214D2-0000-0000-C000-000000000046*/
DEFINE_GUID(
	 CGID_ShellServiceObject
	,0x000214D2L
	,0x0000
	,0x0000
	,0xC0, 0x00
	,0x00, 0x00, 0x00, 0x00, 0x00, 0x46
);

/* IID_IOleCommandTarget:		  B722BCCB-4E68-101B-A2BC-00AA00404770*/
DEFINE_GUID(
	 IID_IOleCommandTarget
	,0xB722BCCBL
	,0x4E68
	,0x101B
	,0xA2, 0xBC
	,0x00, 0xAA, 0x00, 0x40, 0x47, 0x70
);

struct IOleCommandTarget;

typedef struct IOleCommandTarget IOleCommandTarget, *PIOleCommandTarget;

typedef struct IOleCommandTargetVtbl
{
    HRESULT (__stdcall *QueryInterface)( 
		PIOleCommandTarget, REFIID, PVOID*
	);
    ULONG (__stdcall *AddRef)(PIOleCommandTarget);
    ULONG (__stdcall *Release)(PIOleCommandTarget);
    HRESULT (__stdcall *QueryStatus)( 
		PIOleCommandTarget, const GUID*, ULONG, PVOID, PVOID
	);
    HRESULT (__stdcall *Exec)( 
		PIOleCommandTarget, const GUID*, DWORD, DWORD, VARIANT*, VARIANT*
	);
	//
} IOleCommandTargetVtbl, *PIOleCommandTargetVtbl;

struct IOleCommandTarget
{
	const PIOleCommandTargetVtbl lpVtbl;
};
#endif /* __MINGW32__ */


/*---Shell Service Object List Struct-----------------------------------------*/
typedef struct _SSOBJECTLIST
{
	IOleCommandTarget *object;
	//
	struct _SSOBJECTLIST *pNext;
	//
}SSOBJECTLIST, *PSSOBJECTLIST;


/*---Global Variables---------------------------------------------------------*/
static PSSOBJECTLIST g_ssoList;
static BOOL g_bInit;
/*----------------------------------------------------------------------------*/


VOID addAndExecuteSSO(LPCTSTR pszData, DWORD cbData)
{
	if(g_bInit)
	{
		WCHAR wszCLSID[40];
		CLSID clsid;
		IOleCommandTarget *pCmdTarget = NULL;
		HRESULT hr;

#	ifdef _UNICODE
		wcsncpy(wszCLSID, pszData, 40);
		wszCLSID[39] = 0;
#	else
		MultiByteToWideChar(
			CP_ACP,
			0,
			pszData,
			cbData,
			wszCLSID,
			40
		);
#	endif

		CLSIDFromString(wszCLSID, &clsid);

		hr = CoCreateInstance(
			&clsid,
			NULL,
			CLSCTX_INPROC_SERVER | CLSCTX_INPROC_HANDLER,
			&IID_IOleCommandTarget,
			(void **)&pCmdTarget
		);

		if(SUCCEEDED(hr))
		{
			PSSOBJECTLIST entry;
			
			entry = (PSSOBJECTLIST)malloc(sizeof(SSOBJECTLIST));

			if(NULL != entry)
			{
				pCmdTarget->lpVtbl->Exec(
					pCmdTarget,
					&CGID_ShellServiceObject,
					2,
					0,
					NULL,
					NULL
				);

				entry->object = pCmdTarget;
				entry->pNext = g_ssoList;
				g_ssoList = entry;
			}
		}
	}

	return;
}

/*--------LoadShellServiceObjects-----------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
VOID loadShellServiceObjects(
	VOID
)
{
	OSVERSIONINFO oi;
	HRESULT hResult;

	if(NULL != g_ssoList)
	{
		return;
	}

	/* OS version Information */
	memset(&oi, 0 , sizeof(OSVERSIONINFO));
	oi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

	if(
		0 == GetVersionEx(&oi)
		||
		VER_PLATFORM_WIN32_NT != oi.dwPlatformId
		||
		5 > oi.dwMajorVersion
	)
	{
		return;
	}

	/* If we are on 2K/XP, load up shell objects */

	hResult = CoInitialize(NULL);

	/*--------------------------------------------------------------------------
	 Initialize COM interface
	--------------------------------------------------------------------------*/
	if(S_OK == hResult || S_FALSE == hResult)
	{
		HKEY hkeyServices;
		LONG lErrorCode;
		INT nIndex = 0;

		g_bInit = TRUE;

		lErrorCode = RegOpenKeyEx(
			HKEY_LOCAL_MACHINE,
			REGSTR_PATH_SHELLSERVICEOBJECTDELAYED,
			0,
			KEY_READ,
			&hkeyServices
		);

		/*----------------------------------------------------------------------
		 Retrieve all objects from registry list
		----------------------------------------------------------------------*/
		while(ERROR_SUCCESS == lErrorCode)
		{
			_TCHAR szValueName[32];
			_TCHAR szData[40];
			DWORD cbValueName = 32;
			DWORD cbData = 40;
			DWORD dwDataType;

			lErrorCode = RegEnumValue(
				hkeyServices,
				nIndex++,
				szValueName,
				&cbValueName,
				NULL,
				&dwDataType,
				(LPBYTE)szData,
				&cbData
			);

			if(ERROR_SUCCESS == lErrorCode)
			{
				addAndExecuteSSO(szData, cbData);
			}
		}

		RegCloseKey(hkeyServices);

		if(NULL == g_ssoList)
		{
			g_bInit = FALSE;
			CoUninitialize();
		}
	}

	return;
}


/*--------UnloadShellServiceObjects---------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
VOID unloadShellServiceObjects( VOID )
{
	g_bInit = FALSE;

	if(NULL != g_ssoList)
	{
		PSSOBJECTLIST pNext;

		while(NULL != g_ssoList)
		{
			pNext = g_ssoList->pNext;

			g_ssoList->object->lpVtbl->Exec(
				g_ssoList->object,
				&CGID_ShellServiceObject,
				3,
				0,
				NULL,
				NULL
			);

			g_ssoList->object->lpVtbl->Release(g_ssoList->object);

			free(g_ssoList);

			g_ssoList = pNext;
		}

		/*----------------------------------------------------------------------
		 Uninitialize COM interface
		----------------------------------------------------------------------*/
		CoUninitialize();
	}

	return;
}

/*--------End Of File---------------------------------------------------------*/
